/*
 * NucleotideSNP.h
 *
 *  Created on: 8 oct. 2012
 *      Author: abernard
 */

#ifndef ALLELEFREQUENCY_H_
#define ALLELEFREQUENCY_H_

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/Seq/GeneticCode/StandardGeneticCode.h>

// PopPhyl Project Developments
#include "../Seq/Alphabet/CustomNucleicAlphabet.h"
#include "Container/PairedSequenceContainer.h"

namespace ppc {

	class AlleleFrequency {

		protected:
			// Protected attributes
			unsigned int _number_of_allele;
			unsigned int _nb_typed;

			int _allele_1_code;
			int _allele_2_code;
			int _allele_3_code;

			unsigned int _allele_2_rank;
			unsigned int _nb_observed_heterozygosity;

			unsigned int _allele_1_occurence;
			unsigned int _allele_2_occurence;
			unsigned int _allele_3_occurence;

			int _GC_absolute_frequency;
			double _GC_relative_frequency;

			bool _bi_allelic;

			// Protected methods
			void _updateAlleleData(int currentPositionIntCode, unsigned int index);

		public:
			// Constructor with simple PolymorphismSequenceContainer
			AlleleFrequency(bpp::PolymorphismSequenceContainer* ingroupContainer, int codonSiteNumber, int positionNumber, const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet);

			// Constructor with PairedSequenceContainer
			AlleleFrequency(ppc::PairedSequenceContainer* pairContainer, int codonSiteNumber, int positionNumber, const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet);

			// Destructor
			virtual ~AlleleFrequency();

			// Accessor
			unsigned int get_number_of_allele();
			unsigned int get_nb_typed();

			int get_allele_1_code();
			int get_allele_2_code();
			int get_allele_3_code();

			unsigned int get_allele_2_rank();
			unsigned int get_nb_observed_heterozygosity();

			unsigned int get_allele_1_occurence();
			unsigned int get_allele_2_occurence();
			unsigned int get_allele_3_occurence();

			int get_GC_absolute_frequency();
			double get_GC_relative_frequency();

			bool is_bi_allelic();

			// Computing methods
			void determine_GC_frequencies();
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* ALLELEFREQUENCY_H_ */
