/*
 * PairedSequenceContainer.cpp
 *
 *  Created on: 5 nov. 2012
 *      Author: abernard
 */

#include "PairedSequenceContainer.h"

// PopPhyl classes
#include "../Tools/PolymorphismSequenceContainerAddons.h"

using namespace std;
using namespace bpp;
using namespace ppc;

/*-------------------------------*/
/*          Constructor          */
/*-------------------------------*/

PairedSequenceContainer::PairedSequenceContainer(bpp::PolymorphismSequenceContainer* selectedContainer, const bpp::StandardCodonAlphabet* codonAlphabet, std::string nameElementSeparator, unsigned int maxNameElementToRetrieve):
		_current_position(0), _nb_pair(0), _pairs(0), _pair_names(0) {

	// Temporary variables
	string lastSecondAllele("");

	// Identify paired sequences
	for (unsigned int i = 0; i < selectedContainer->getNumberOfSequences(); i++) {
		for (unsigned int j = i+1; j < selectedContainer->getNumberOfSequences(); j++) {

			if (lastSecondAllele == selectedContainer->getName(i)) continue;

			if (isaPair(selectedContainer->getName(i), selectedContainer->getName(j))) {

				// Create a distinct container for the new individual
				PolymorphismSequenceContainer* newPair = new PolymorphismSequenceContainer(codonAlphabet);

				// Add a general comment to the new container
				bpp::Comments containerComment = PolymorphismSequenceContainerAddons::getNameElementsFromContainer(selectedContainer, "|", i, 3);
				newPair->setGeneralComments(containerComment);

				// Store pair name
				_pair_names.push_back(PolymorphismSequenceContainerAddons::getIndividualNameFromContainer(selectedContainer, i));

				// Add the 2 sequence to the container
				newPair->addSequence(selectedContainer->getSequence(i));
				newPair->addSequence(selectedContainer->getSequence(j));

				// Add the complete container to the global vector
				_pairs.push_back(newPair);

				lastSecondAllele = selectedContainer->getName(j);
				break;
			}
		}
	}

	_nb_pair = _pairs.size();
}

/*--------------------------------*/
/*           Destructor           */
/*--------------------------------*/

PairedSequenceContainer::~PairedSequenceContainer() {

	// Delete all pointers to PSC
	if (_nb_pair > 0) {
		for (unsigned int i = 0; i < _nb_pair; i++) {
			delete(_pairs[i]);
		}
	}
}

/*---------------------------------*/
/*            Accessors            */
/*---------------------------------*/

// GET
unsigned int PairedSequenceContainer::get_current_position() { return _current_position; }

unsigned int PairedSequenceContainer::get_nb_pair() { return _nb_pair; }

vector<string> PairedSequenceContainer::get_pair_names() { return _pair_names; }


/*------------------------------------*/
/*           Public methods           */
/*------------------------------------*/

bpp::PolymorphismSequenceContainer* PairedSequenceContainer::next_pair() throw (Exception){

	if (_current_position + 1 > _nb_pair) throw Exception("There is no more pair in the container");

	return PairedSequenceContainer::get_pair(_current_position++);
}

/******************************************************************************/

bpp::PolymorphismSequenceContainer* PairedSequenceContainer::get_pair(unsigned int pairIndex) throw (Exception) {

	if (pairIndex  >= _nb_pair) throw Exception("Requested pair doesn't exist the container");

	return _pairs[pairIndex];
}

/******************************************************************************/

void PairedSequenceContainer::reset_index_position() { _current_position = 0; }


/*-------------------------------------*/
/*          Protected methods          */
/*-------------------------------------*/

bool PairedSequenceContainer::isaPair(string sequenceNameA, string sequenceNameB) {

	// Initializations
	string subSequenceNameA(""), subSequenceNameB("");

	// Extract the portion of the name before the last underscore (Sequence A)
	for (unsigned int i = sequenceNameA.size() - 1; i > 0; i--) {
		if (sequenceNameA[i] == '_') {
			subSequenceNameA = sequenceNameA.substr(0,i);
			break;
		}
	}
	if (subSequenceNameA == "") return false;

	// Extract the portion of the name before the last underscore (Sequence B)
	for (unsigned j = sequenceNameB.size() - 1; j > 0; j--) {
		if (sequenceNameB[j] == '_') {
			subSequenceNameB = sequenceNameB.substr(0,j);
			break;
		}
	}
	if (subSequenceNameB == "") return false;

	// Compare sub sequence's names
	if (subSequenceNameA == subSequenceNameB) {
		return true;
	} else {
		return false;
	}
}
