/*
 * PairedSequenceContainer.h
 *
 *  Created on: 5 nov. 2012
 *      Author: abernard
 */

#ifndef PAIREDSEQUENCECONTAINER_H_
#define PAIREDSEQUENCECONTAINER_H_

// Basic C++ classes
#include <string>
#include <vector>

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/Exceptions.h>

namespace ppc {

	class PairedSequenceContainer {

		protected:
			// Protected attributes
			unsigned int _current_position;
			unsigned int _nb_pair;

			std::vector<bpp::PolymorphismSequenceContainer*> _pairs;
			std::vector<std::string> _pair_names;

			// Protected methods
			bool isaPair(std::string sequenceNameA, std::string sequenceNameB);

		public:
			// Constructor
			PairedSequenceContainer(bpp::PolymorphismSequenceContainer* selectedContainer, const bpp::StandardCodonAlphabet* codonAlphabet, std::string nameElementSeparator, unsigned int maxNameElementToRetrieve);

			// Destructor
			virtual ~PairedSequenceContainer();

			// Accessors
			unsigned int get_current_position();
			unsigned int get_nb_pair();
			std::vector<std::string> get_pair_names();

			// Other public methods
			bpp::PolymorphismSequenceContainer* next_pair() throw (bpp::Exception);
			bpp::PolymorphismSequenceContainer* get_pair(unsigned int pairIndex) throw (bpp::Exception);
			void reset_index_position();


	};

} //end of namespace PopPhyl Commons (ppc).

#endif /* PAIREDSEQUENCECONTAINER_H_ */
