/*
 * DistanceBetweenIndividuals.h
 *
 *  Created on: 28 nov. 2012
 *      Author: abernard
 */

#ifndef DISTANCEBETWEENINDIVIDUALS_H_
#define DISTANCEBETWEENINDIVIDUALS_H_

// Basic C++ classes
#include <string>

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/Seq/Alphabet/NucleicAlphabet.h>

// PopPhyl Project Developments
#include "Container/PairedSequenceContainer.h"

namespace ppc {

	class DistanceBetweenIndividuals {

		protected:
			std::string _first_individual_name;
			std::string _second_individual_name;
			std::string _pair_name;

			double _distance;
			int _nb_resolved_positions;

			void calculateDistance();

		public:
			// Constructors
			DistanceBetweenIndividuals(std::string individualName1, std::string individualName2);

			DistanceBetweenIndividuals(bpp::PolymorphismSequenceContainer* individual1, bpp::PolymorphismSequenceContainer* individual2);

			DistanceBetweenIndividuals(ppc::PairedSequenceContainer* pairContainer, unsigned int pairIndex1, unsigned int pairIndex2);

			// Destructor
			virtual ~DistanceBetweenIndividuals();

			// Accessors
			std::string get_first_individual_name();
			std::string get_second_individual_name();
			std::string get_pair_name();

			double get_distance();
			int get_nb_resolved_positions();

			void set_distance(double computedDistance);
			void set_nb_resolved_positions(int determinedNbResolvedPositions);

			// Computing methods
			void computeDistance(bpp::PolymorphismSequenceContainer* individual1, bpp::PolymorphismSequenceContainer* individual2,
					const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet, bool debugMode = false);

			void computeDistanceWithPair(ppc::PairedSequenceContainer* pairContainer, unsigned int pairIndex1, unsigned int pairIndex2,
					const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet, bool debugMode = false);
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* DISTANCEBETWEENINDIVIDUALS_H_ */
