/*
 * Polymorphism.cpp
 *
 *  Created on: 25 oct. 2012
 *      Author: abernard
 */

#include "Polymorphism.h"

// Bio++ classes
#include <Bpp/Seq/Container/SiteContainerTools.h>
#include <Bpp/PopGen/PolymorphismSequenceContainerTools.h>
#include <Bpp/PopGen/SequenceStatistics.h>

using namespace std;
using namespace bpp;
using namespace ppc;

/*-------------------------------*/
/*          Constructor          */
/*-------------------------------*/

Polymorphism::Polymorphism(PolymorphismSequenceContainer* groupContainer): _pi_non_synonymous(0), _pi_synonymous(0), _mean_non_synonymous_sites_number(0),
_mean_synonymous_sites_number(0), _number_of_sequences(0), _number_of_complete_sites(0), _piN(-1.), _piS(-1.), _internal_psc_pointer(groupContainer) {
	// Nothing to do here, everything is done with the initialization list
}


/*--------------------------------*/
/*           Destructor           */
/*--------------------------------*/

Polymorphism::~Polymorphism() {
	// Unlink the pointer
	_internal_psc_pointer = 0;
}


/*---------------------------------*/
/*            Accessors            */
/*---------------------------------*/

// GET
double Polymorphism::get_pi_non_synonymous() { return _pi_non_synonymous; }
double Polymorphism::get_pi_synonymous() { return _pi_synonymous; }
double Polymorphism::get_mean_non_synonymous_sites_number() { return _mean_non_synonymous_sites_number; }
double Polymorphism::get_mean_synonymous_sites_number() { return _mean_synonymous_sites_number; }

unsigned int Polymorphism::get_number_of_sequences() { return _number_of_sequences; }
unsigned int Polymorphism::get_number_of_complete_sites() { return _number_of_complete_sites; }

double Polymorphism::get_piN() { return _piN; }
double Polymorphism::get_piS() { return _piS; }


/*------------------------------------*/
/*           Public methods           */
/*------------------------------------*/

/*       Computing methods       */

void Polymorphism::determine_all_attributes(StandardGeneticCode* stdGeneticCode, unsigned int minimumNumberOfCodon, double kappa) {

	determine_pi_non_synonymous(stdGeneticCode);
	determine_pi_synonymous(stdGeneticCode);
	determine_mean_non_synonymous_sites_number(stdGeneticCode, kappa);
	determine_mean_synonymous_sites_number(stdGeneticCode, kappa);

	determine_number_of_sequences();
	determine_number_of_complete_sites();

	determine_piN(minimumNumberOfCodon);
	determine_piS(minimumNumberOfCodon);
}

/******************************************************************************/

void Polymorphism::determine_pi_non_synonymous(StandardGeneticCode* stdGeneticCode) {
	_pi_non_synonymous = SequenceStatistics::piNonSynonymous(*_internal_psc_pointer, *stdGeneticCode, false);
}

/******************************************************************************/

void Polymorphism::determine_pi_synonymous(StandardGeneticCode* stdGeneticCode) {
	_pi_synonymous = SequenceStatistics::piSynonymous(*_internal_psc_pointer, *stdGeneticCode, false);
}


void Polymorphism::determine_mean_non_synonymous_sites_number(StandardGeneticCode* stdGeneticCode, double kappa) {
	_mean_non_synonymous_sites_number =SequenceStatistics::meanNonSynonymousSitesNumber(*_internal_psc_pointer, *stdGeneticCode, kappa);
}

/******************************************************************************/

void Polymorphism::determine_mean_synonymous_sites_number(StandardGeneticCode* stdGeneticCode, double kappa) {
	_mean_synonymous_sites_number = SequenceStatistics::meanSynonymousSitesNumber(*_internal_psc_pointer, *stdGeneticCode, kappa);
}

/******************************************************************************/

void Polymorphism::determine_number_of_sequences() {
	_number_of_sequences = _internal_psc_pointer->getNumberOfSequences();
}

/******************************************************************************/

void Polymorphism::determine_number_of_complete_sites() {
	_number_of_complete_sites = PolymorphismSequenceContainerTools::getNumberOfCompleteSites(*_internal_psc_pointer, false);
}

/******************************************************************************/

void Polymorphism::determine_piN(unsigned int minimumNumberOfCodon) {

	if (_number_of_complete_sites >= minimumNumberOfCodon) {
		_piN = _pi_non_synonymous / _mean_non_synonymous_sites_number;
	}
}

/******************************************************************************/

void Polymorphism::determine_piS(unsigned int minimumNumberOfCodon) {

	if (_number_of_complete_sites >= minimumNumberOfCodon) {
		_piS = _pi_synonymous / _mean_synonymous_sites_number;
	}
}
