/*
 * Polymorphism.h
 *
 *  Created on: 25 oct. 2012
 *      Author: abernard
 */

#ifndef POLYMORPHISM_H_
#define POLYMORPHISM_H_

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/GeneticCode/StandardGeneticCode.h>

namespace ppc {

	class Polymorphism {

		protected:
			double _pi_non_synonymous;
			double _pi_synonymous;
			double _mean_non_synonymous_sites_number;
			double _mean_synonymous_sites_number;

			unsigned int _number_of_sequences;
			unsigned int _number_of_complete_sites;

			double _piN;
			double _piS;

			bpp::PolymorphismSequenceContainer* _internal_psc_pointer;

		public:
			// Constructor
			Polymorphism(bpp::PolymorphismSequenceContainer* groupContainer);

			// Destructor
			virtual ~Polymorphism();

			// Computing methods
			void determine_all_attributes(bpp::StandardGeneticCode* stdGeneticCode, unsigned int minimumNumberOfCodon, double kappa);

			void determine_pi_non_synonymous(bpp::StandardGeneticCode* stdGeneticCode);
			void determine_pi_synonymous(bpp::StandardGeneticCode* stdGeneticCode);
			void determine_mean_non_synonymous_sites_number(bpp::StandardGeneticCode* stdGeneticCode, double kappa);
			void determine_mean_synonymous_sites_number(bpp::StandardGeneticCode* stdGeneticCode, double kappa);

			void determine_number_of_sequences();
			void determine_number_of_complete_sites();

			void determine_piN(unsigned int minimumNumberOfCodon);
			void determine_piS(unsigned int minimumNumberOfCodon);

			// Accessors
			double get_pi_non_synonymous();
			double get_pi_synonymous();
			double get_mean_non_synonymous_sites_number();
			double get_mean_synonymous_sites_number();

			unsigned int get_number_of_sequences();
			unsigned int get_number_of_complete_sites();

			double get_piN();
			double get_piS();
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* POLYMORPHISM_H_ */
