/*
 * Population.h
 *
 *  Created on: 24 oct. 2012
 *      Author: abernard
 */

#ifndef POPULATION_H_
#define POPULATION_H_

// Basic C++ classes
#include <iostream>
#include <string>
#include <vector>

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Exceptions.h>

namespace ppc {

	class Population {

		protected:
			// Protected attributes
			std::vector<std::string> _population_names;
			std::vector< std::vector<std::string> > _population_list;
			std::vector<std::string> _duplicates;

			unsigned int _number_of_population;

			// Protected methods
			std::vector<std::string> convert_file_into_vector_of_string(std::string populationFile);
			void build_population_list(std::vector<std::string> AllPopdata);



		public:
			// Constructors
			Population(std::string populationFile);
			Population(std::vector< bpp::PolymorphismSequenceContainer* >& allDatasets, std::string ingroupSpeciesName);

			// Destructor
			virtual ~Population();

			// Accessor
			std::vector<std::string> get_population_names();
			std::vector< std::vector<std::string> > get_population_list();
			std::vector<std::string> get_duplicates();

			unsigned int get_number_of_population();

			// Public methods
			int get_population_id(std::string individualName) throw (bpp::Exception);
			std::string get_population_name(std::string individualName);
			std::vector<std::string> get_all_individuals();
			std::vector<std::string> get_all_couples();

			unsigned int get_number_of_individuals();

			bool check_for_duplicates();

			void display_populations(bool generated = false);
			void display_duplicates();
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* POPULATION_H_ */
