/*
 * PopulationDiversification.h
 *
 *  Created on: 14 nov. 2012
 *      Author: abernard
 */

#ifndef POPULATIONDIVERSIFICATION_H_
#define POPULATIONDIVERSIFICATION_H_

// Basic C++ classes
#include <vector>
#include <map>

// Bio++ classes
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>

// PopPhyl Project Developments
#include "../Seq/Alphabet/CustomNucleicAlphabet.h"
#include "Population.h"

namespace ppc {

	class PopulationDiversification {

		protected:
			std::vector<int> _differenceWithin;
			std::vector<int> _totalWithin;

			std::vector< std::vector<int> > _differenceBetween;
			std::vector< std::vector<int> > _totalBetween;

			std::map<int, double> _heterozygosityWithinPops;

			std::vector< std::vector<double> > _globalHeteroWithin;
			std::vector< std::vector<double> > _globalHeteroBetween;
			std::vector< std::vector<double> > _Fst;

			ppc::Population* _internal_pop_pointer;

			// Computing methods
			void _collect_differences(bpp::PolymorphismSequenceContainer* selectedContainer, const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet);

		public:
			// Constructor
			PopulationDiversification(bpp::PolymorphismSequenceContainer* selectedContainer, ppc::Population* populationData, const bpp::StandardCodonAlphabet* codonAlphabet, const bpp::NucleicAlphabet* nucleoAlphabet);

			// Destructor
			virtual ~PopulationDiversification();

			// Computing methods
			void determine_heterozygosities_only();
			void determine_Fst();

			// Accessors - Get all
			std::vector<int> get_all_differenceWithin();
			std::vector<int> get_all_totalWithin();

			std::vector< std::vector<int> > get_all_differenceBetween();
			std::vector< std::vector<int> > get_all_totalBetween();

			std::map<int, double> get_all_heterozygosityWithinPops();

			std::vector< std::vector<double> > get_all_globalHeteroWithin();
			std::vector< std::vector<double> > get_all_globalHeteroBetween();

			std::vector< std::vector<double> > get_all_Fst();

			// Accessors - Get specific
			int get_specific_differenceWithin(int popIndex);
			int get_specific_totalWithin(int popIndex);

			int get_specific_differenceBetween(int popIndex1, int popIndex2);
			int get_specific_totalBetween(int popIndex1, int popIndex2);

			double get_specific_heterozygosityWithinPop(int popIndex);

			double get_specific_globalHeteroWithin(int popIndex1, int popIndex2);
			double get_specific_globalHeteroBetween(int popIndex1, int popIndex2);

			double get_specific_Fst(int popIndex1, int popIndex2);
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* POPULATIONDIVERSIFICATION_H_ */
