/*
 * SFS.h
 *
 *  Created on: 22 oct. 2012
 *      Author: abernard
 */

#ifndef SFS_H_
#define SFS_H_

// Basic C++ classes
#include <vector>

namespace ppc {

	class SFS {

		protected:
			// Restricted attributes
			std::vector <double> _Hypergeometric_draw;
			std::vector <double> _sumlnk;

			unsigned int _sample_size;
			bool _needFolding;

			// Restricted methods
			void compute_sumlnk (unsigned int vectorSize);
			double lnCnk(int n, int k);
			double hypergeometric_prob(int k, int n, int pa, int tot);

		public:
			// Constructors
			SFS(unsigned int maxVectorSize, unsigned int sampleSize, std::vector< std::vector<double> >& originalMatrix, bool requireFolding);
			SFS(unsigned int maxVectorSize, unsigned int sampleSize, std::vector<double>& hypergeometricDraw);

			// Destructor
			virtual ~SFS();

			// Accessors
			void change_sample_size(unsigned int newSampleSize);
			unsigned int get_sample_size();

			std::vector <double> get_Hypergeometric_draw();

			// Simple public methods
			std::vector < std::vector<double> > fold_square_matrix(std::vector< std::vector<double> >& squareMatrix, unsigned int matrixSideSize);

			// Hypergeometric drawing related methods
			void folded_hypergeometric_drawing(std::vector < std::vector <double> >& foldedMatrix, unsigned int matrixSideSize);
			void unfolded_hypergeometric_drawing(std::vector < std::vector <double> >& unfoldedMatrix, unsigned int matrixSideSize);
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* SFS_H_ */
