/*
 * SFSneutral.cpp
 *
 *  Created on: 22 oct. 2012
 *      Author: abernard
 */

#include "SFSneutral.h"

// Basic C++ classes
#include <math.h>

using namespace std;
using namespace ppc;

/*-------------------------------*/
/*          Constructor          */
/*-------------------------------*/

SFSneutral::SFSneutral(unsigned int maxVectorSize, unsigned int sampleSize, std::vector< std::vector<double> >& neutralMatrix, bool requireFolding):
		SFS(maxVectorSize, sampleSize, neutralMatrix, requireFolding), _total_number_of_synonymous_SNP(0.), _average_allele_frequency_S(0.), _piS(0.), _thetaS(0.), _tajimasD_S(0.) {
	// Nothing to do here: the parent class constructor and the initialization list do all the work
}

/******************************************************************************/

SFSneutral::SFSneutral(unsigned int maxVectorSize, unsigned int sampleSize, std::vector<double>& neutralHypergeometricDraw):
		SFS(maxVectorSize, sampleSize, neutralHypergeometricDraw), _total_number_of_synonymous_SNP(0.), _average_allele_frequency_S(0.), _piS(0.), _thetaS(0.), _tajimasD_S(0.) {
	// Nothing to do here: the parent class constructor and the initialization list do all the work
}


/*--------------------------------*/
/*           Destructor           */
/*--------------------------------*/

SFSneutral::~SFSneutral() {
	// TODO Auto-generated destructor stub
}


/*--------------------------------*/
/*            Accessor            */
/*--------------------------------*/

// GET
double SFSneutral::get_total_number_of_synonymous_SNP() { return _total_number_of_synonymous_SNP; }
double SFSneutral::get_average_allele_frequency_S() { return _average_allele_frequency_S; }
double SFSneutral::get_piS() { return _piS; }
double SFSneutral::get_thetaS() { return _thetaS; }
double SFSneutral::get_tajimasD_S() { return _tajimasD_S; }


/*------------------------------------*/
/*           Public methods           */
/*------------------------------------*/

/*       Computing methods       */

void SFSneutral::determine_all_attributes() {

	determine_total_number_of_synonymous_SNP();
	determine_average_allele_frequency_S();
	determine_piS();
	determine_thetaS();
	determine_tajimasD_S();
}

/******************************************************************************/

void SFSneutral::determine_total_number_of_synonymous_SNP() {

	for (unsigned int k = 1; k <= _sample_size / 2; k++) {
		_total_number_of_synonymous_SNP += _Hypergeometric_draw[k];
	}
}

/******************************************************************************/

void SFSneutral::determine_average_allele_frequency_S() {

	for (unsigned int k = 1; k <= _sample_size / 2; k++) {
		_average_allele_frequency_S += k * _Hypergeometric_draw[k];
	}

	_average_allele_frequency_S /= (_total_number_of_synonymous_SNP * _sample_size);
}

/******************************************************************************/

void SFSneutral::determine_piS() {

	for (unsigned int i = 1; i < _sample_size; i++) {
		_piS += _Hypergeometric_draw[i] * i * (_sample_size - i);
	}

	_piS *= 2;
	_piS /= (_sample_size * (_sample_size - 1.));
}

/******************************************************************************/

void SFSneutral::determine_thetaS() {
	// Initializations
	double a1(0.);

	// Prepare data for theta calculation
	for (unsigned int i = 1; i < _sample_size; i++) {
		a1 += 1. / (double)i;
	}

	_thetaS = _total_number_of_synonymous_SNP / a1;
}

/******************************************************************************/

void SFSneutral::determine_tajimasD_S() {
	// Initializations
	double a1(0.), a2(0.);
	double b1, b2, c1, c2, e1, e2;

	// Prepare data for theta calculation
	for (unsigned int i = 1; i < _sample_size; i++) {
		a1 += 1. / (double)i;
		a2 += 1. / (double)(i * i);
	}

	// a1n = a1 + 1. / _sample_size;
	b1 = (_sample_size + 1.) / (3. * (_sample_size-1.));
	b2 = (2. * (_sample_size * _sample_size + _sample_size + 3.)) / (9. * _sample_size * (_sample_size-1.));
	c1 = b1-(1. / a1);
	c2 = b2-((_sample_size + 2.) / (a1 * _sample_size)) + (a2 / (a1 * a1));
	e1 = c1 / a1;
	e2 = c2 / ((a1 * a1) + a2);

	_tajimasD_S = (_piS - _thetaS) / (sqrt(e1 * _total_number_of_synonymous_SNP + e2 * _total_number_of_synonymous_SNP * (_total_number_of_synonymous_SNP - 1.)));
}
