/*
 * SFSselected.cpp
 *
 *  Created on: 22 oct. 2012
 *      Author: abernard
 */

#include "SFSselected.h"

#include <math.h>

using namespace std;
using namespace ppc;

/*-------------------------------*/
/*          Constructor          */
/*-------------------------------*/

SFSselected::SFSselected(unsigned int maxVectorSize, unsigned int sampleSize, std::vector< std::vector<double> >& selectedMatrix, bool requireFolding):
		SFS(maxVectorSize, sampleSize, selectedMatrix, requireFolding), _total_number_of_non_synonymous_SNP(0.), _average_allele_frequency_N(0.), _piN(0.), _thetaN(0.), _tajimasD_N(0.) {
	// Nothing to do here: the parent class constructor and the initialization list do all the work
}

/******************************************************************************/

SFSselected::SFSselected(unsigned int maxVectorSize, unsigned int sampleSize, std::vector<double>& selectedHypergeometricDraw):
		SFS(maxVectorSize, sampleSize, selectedHypergeometricDraw), _total_number_of_non_synonymous_SNP(0.), _average_allele_frequency_N(0.), _piN(0.), _thetaN(0.), _tajimasD_N(0.) {
	// Nothing to do here: the parent class constructor and the initialization list do all the work
}


/*--------------------------------*/
/*           Destructor           */
/*--------------------------------*/

SFSselected::~SFSselected() {
	// TODO Auto-generated destructor stub
}


/*--------------------------------*/
/*            Accessor            */
/*--------------------------------*/

// GET
double SFSselected::get_total_number_of_non_synonymous_SNP() { return _total_number_of_non_synonymous_SNP; }
double SFSselected::get_average_allele_frequency_N() {return _average_allele_frequency_N; }
double SFSselected::get_piN() {return _piN; }
double SFSselected::get_thetaN() { return _thetaN; }
double SFSselected::get_tajimasD_N() { return _tajimasD_N; }


/*------------------------------------*/
/*           Public methods           */
/*------------------------------------*/

/*       Computing methods       */

void SFSselected::determine_all_attributes() {

	determine_total_number_of_non_synonymous_SNP();
	determine_average_allele_frequency_N();
	determine_piN();
	determine_thetaN();
	determine_tajimasD_N();
}

/******************************************************************************/

void SFSselected::determine_total_number_of_non_synonymous_SNP() {

	for (unsigned int k = 1; k <= _sample_size / 2; k++) {
		_total_number_of_non_synonymous_SNP += _Hypergeometric_draw[k];
	}
}

/******************************************************************************/

void SFSselected::determine_average_allele_frequency_N() {

	for (unsigned int k = 1; k <= _sample_size / 2; k++) {
		_average_allele_frequency_N += k * _Hypergeometric_draw[k];
	}

	_average_allele_frequency_N /= (_total_number_of_non_synonymous_SNP * _sample_size);
}

/******************************************************************************/

void SFSselected::determine_piN() {

	for (unsigned int i = 1; i <= _sample_size / 2; i++) {
		_piN += _Hypergeometric_draw[i] * i * (_sample_size - i);
	}

	//if (_sample_size % 2 == 0) _piN += (_Hyper_SFS_selected[_sample_size / 2] * _sample_size * _sample_size / 4.);
	_piN *= 2;
	_piN /= (_sample_size * (_sample_size - 1.));
}

/******************************************************************************/

void SFSselected::determine_thetaN() {
	// Initializations
	double a1(0.);

	// Prepare data for theta calculation
	for (unsigned int i = 1; i < _sample_size; i++) {
		a1 += 1. / (double)i;
	}

	_thetaN = _total_number_of_non_synonymous_SNP / a1;
}

/******************************************************************************/

void SFSselected::determine_tajimasD_N() {
	// Initializations
	double a1(0.), a2(0.);
	double b1, b2, c1, c2, e1, e2;

	// Prepare data for theta calculation
	for (unsigned int i = 1; i < _sample_size; i++) {
		a1 += 1. / (double)i;
		a2 += 1. / (double)(i * i);
	}

	// a1n = a1 + 1. / _sample_size;
	b1 = (_sample_size + 1.) / (3. * (_sample_size-1.));
	b2 = (2. * (_sample_size * _sample_size + _sample_size + 3.)) / (9. * _sample_size * (_sample_size-1.));
	c1 = b1-(1. / a1);
	c2 = b2-((_sample_size + 2.) / (a1 * _sample_size)) + (a2 / (a1 * a1));
	e1 = c1 / a1;
	e2 = c2 / ((a1 * a1) + a2);

	_tajimasD_N = (_piN - _thetaN) / (sqrt(e1 * _total_number_of_non_synonymous_SNP + e2 * _total_number_of_non_synonymous_SNP * (_total_number_of_non_synonymous_SNP - 1.)));
}
