/*
 * SelectionIndicators.h
 *
 *  Created on: 22 oct. 2012
 *      Author: abernard
 */

#ifndef SELECTIONINDICATORS_H_
#define SELECTIONINDICATORS_H_

// Basic C++ classes
#include <string>

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/Seq/GeneticCode/StandardGeneticCode.h>

namespace ppc {

	class SelectionIndicators {

		protected:
			double _nN;
			double _nS;
			double _fixN;
			double _fixS;
			double _fixSnS_ratio;

			unsigned int _number_of_sequences;
			unsigned int _number_of_sites;
			unsigned int _number_of_complete_sites;

			bpp::PolymorphismSequenceContainer* _consensus;

		public:
			// Constructors
			SelectionIndicators(bpp::PolymorphismSequenceContainer* ingroupContainer, bpp::PolymorphismSequenceContainer* outgroupContainer, const bpp::StandardCodonAlphabet* codonAlphabet,
					std::string ingroupConsensusName, std::string outgroupConsensusName);

			SelectionIndicators(const bpp::Sequence* ingroupConsensusSequence, const bpp::Sequence* outgroupConsensusSequence, const bpp::StandardCodonAlphabet* codonAlphabet);

			SelectionIndicators(bpp::PolymorphismSequenceContainer* ingroupContainer, const bpp::StandardCodonAlphabet* codonAlphabet, std::string ingroupConsensusName);

			// Destructor
			virtual ~SelectionIndicators();

			// Computing methods
			void determine_all_attributes(bpp::StandardGeneticCode* stdGeneticCode, double kappa);
			void determine_nN(bpp::StandardGeneticCode* stdGeneticCode, double kappa);
			void determine_nS(bpp::StandardGeneticCode* stdGeneticCode, double kappa);
			void determine_fixN(bpp::StandardGeneticCode* stdGeneticCode);
			void determine_fixS(bpp::StandardGeneticCode* stdGeneticCode);
			void determine_fixSnS_ratio();

			void countSequencesAndSites();

			// Accessors
			double get_nN();
			double get_nS();
			double get_fixN();
			double get_fixS();
			double get_fixSnS_ratio();

			unsigned int get_number_of_sequences();
			unsigned int get_number_of_sites();
			unsigned int get_number_of_complete_sites();
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* SELECTIONINDICATORS_H_ */
