/*
 * PolymorphismSequenceContainerAddons.h
 *
 *  Created on: 12 oct. 2012
 *      Author: abernard
 */

#ifndef POLYMORPHISMSEQUENCECONTAINERADDONS_H_
#define POLYMORPHISMSEQUENCECONTAINERADDONS_H_

// Basic C++ classes
#include <string>
#include <vector>
#include <map>

// Bio++ classes
#include <Bpp/PopGen/PolymorphismSequenceContainer.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>

namespace ppc {

	class PolymorphismSequenceContainerAddons {

		public:
			// Destructor
			virtual ~PolymorphismSequenceContainerAddons();

			// Static methods
			static bpp::Site generateValidUnknownSite(bpp::PolymorphismSequenceContainer* currentContainer, const bpp::StandardCodonAlphabet* codonAlphabet);
			static void tagAllSequences (bpp::PolymorphismSequenceContainer* currentContainer, std::string ingroupSpeciesName, bool debugMode = false, std::string outgroupSpeciesName = "none");
			static std::map<unsigned int, unsigned int> getNumberOfMembers (bpp::PolymorphismSequenceContainer*, bool debugMode);
			static std::string getContigNameFromContainer (bpp::PolymorphismSequenceContainer* currentContainer, unsigned int indexToUse);
			static std::string getSpeciesNameFromContainer (bpp::PolymorphismSequenceContainer* currentContainer, unsigned int indexToUse);
			static std::string getIndividualNameFromContainer (bpp::PolymorphismSequenceContainer* currentContainer, unsigned int indexToUse);
			static std::vector<std::string> getNameElementsFromContainer (bpp::PolymorphismSequenceContainer* currentContainer, std::string separator, unsigned int indexToUse, unsigned int maxElementToRetrieve);
			static void removeLeftBadSites (bpp::PolymorphismSequenceContainer* currentContainer, int nbSiteToRemoveOnTheLeft, bool debugMode = false);
			static void removeRightBadSites (bpp::PolymorphismSequenceContainer* currentContainer, int nbSiteToRemoveOnTheRight, bool debugMode = false);
			static bool dealWithBadCodons (bpp::PolymorphismSequenceContainer* currentContainer, const bpp::StandardCodonAlphabet* codonAlphabet, unsigned int stopToleranceZone, bool allowInternalStop, bool debugMode = false);
			static void removeHighlyGappedSites (bpp::PolymorphismSequenceContainer* currentContainer, const bpp::StandardCodonAlphabet* codonAlphabet, unsigned int numberOfIngroupSequence, unsigned int minGapNSite, bool debugMode = false);
			static void removeHighlyGappedSequences (bpp::PolymorphismSequenceContainer* currentContainer, const bpp::StandardCodonAlphabet* codonAlphabet, double maxGapNSeq, bool debugMode = false);
			static void removeGroupFromContainer (bpp::PolymorphismSequenceContainer* currentContainer, unsigned int groupId, bool debugMode = false);
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* POLYMORPHISMSEQUENCECONTAINERADDONS_H_ */
