/*
 * CustomNucleicAlphabet.cpp
 *
 * Created on: 2 nov. 2010
 *      Author: V. Cahais
 *
 * Updated on: 27 nov. 2012 (Frameshift management)
 *  	Edited by: A. Bernard
 */

#include "CustomNucleicAlphabet.h"

// Basic C++ classes
#include <map>

// Bio++ classes
#include <Bpp/Seq/Alphabet/AlphabetState.h>
#include <Bpp/Text/TextTools.h>
#include <Bpp/Utils/MapTools.h>

// Namespaces
using namespace std;
using namespace bpp;
using namespace ppc;

/*--------------------------------*/
/*          Constructors          */
/*--------------------------------*/

CustomNucleicAlphabet::CustomNucleicAlphabet() {

	// Alphabet size definition
	resize(18);

	// Alphabet content definition
	setState( 0, NucleicAlphabetState(-1, "-",  0, "Gap"));
	setState( 1, NucleicAlphabetState( 0, "A",  1, "Adenine"));
	setState( 2, NucleicAlphabetState( 1, "C",  2, "Cytosine"));
	setState( 3, NucleicAlphabetState( 2, "G",  4, "Guanine"));
	setState( 4, NucleicAlphabetState( 3, "T",  8, "Thymine"));
	setState( 5, NucleicAlphabetState( 4, "M",  3, "Adenine or Cytosine"));
	setState( 6, NucleicAlphabetState( 5, "R",  5, "Purine (Adenine or Guanine)"));
	setState( 7, NucleicAlphabetState( 6, "W",  9, "Adenine or Thymine"));
	setState( 8, NucleicAlphabetState( 7, "S",  6, "Cytosine or Guanine"));
	setState( 9, NucleicAlphabetState( 8, "Y", 10, "Pyrimidine (Cytosine or Thymine)"));
	setState(10, NucleicAlphabetState( 9, "K", 12, "Guanine or Thymine"));
	setState(11, NucleicAlphabetState(10, "V",  7, "Adenine or Cytosine or Guanine"));
	setState(12, NucleicAlphabetState(11, "H", 11, "Adenine or Cytosine or Thymine"));
	setState(13, NucleicAlphabetState(12, "D", 13, "Adenine or Guanine or Thymine"));
	setState(14, NucleicAlphabetState(13, "B", 14, "Cytosine or Guanine or Thymine"));
	setState(15, NucleicAlphabetState(14, "!", 15, "Frameshift"));
	setState(16, NucleicAlphabetState(15, "N", 16, "Unresolved base N"));
	setState(17, NucleicAlphabetState(15, "*", 16, "Unresolved base *"));
}


/*------------------------------------*/
/*           Public methods           */
/*------------------------------------*/

vector<int> CustomNucleicAlphabet::getAlias(int state) const throw (BadIntException) {

	// Exceptions
	if(!isIntInAlphabet(state)) throw BadIntException(state, "DNA::getAlias(int): Specified base unknown.");

	// Initializations
	vector<int> v;
	const NucleicAlphabetState& st = getState(state);

	// Get alias
	if (st.getBinaryCode() & 1)	v.push_back(0);
	if (st.getBinaryCode() & 2)	v.push_back(1);
	if (st.getBinaryCode() & 4)	v.push_back(2);
	if (st.getBinaryCode() & 8)	v.push_back(3);

	return v;
}

/******************************************************************************/

vector<string> CustomNucleicAlphabet::getAlias(const string& state) const throw (BadCharException) {

	// Exceptions
	string locstate = TextTools::toUpper(state);
	if(!isCharInAlphabet(locstate)) throw BadCharException(locstate, "DNA::getAlias(int): Specified base unknown.");

	// Initializations
	vector<int> vi = this->getAlias(this->charToInt(state));
	vector<string> v;

	// Get alias
	for (unsigned int i = 0 ; i < vi.size() ; i++) v.push_back(this->intToChar(vi[i]));

	return v;
}

/******************************************************************************/

int CustomNucleicAlphabet::getGeneric(const vector<int>& states) const throw (BadIntException) {

	// Initializations
	char v = 0;

	for (unsigned int i = 0 ; i < states.size() ; ++i) {
		if (!isIntInAlphabet(states[i])) throw BadIntException(states[i], "DNA::getGeneric(const vector<int>& states): Specified base unknown.");
		v |= getState(states[i]).getBinaryCode();
	}

	return getStateByBinCode(v).getNum();
}

/******************************************************************************/

string CustomNucleicAlphabet::getGeneric(const vector<string>& states) const throw (BadCharException) {
	// Initializations
	vector<int> vi;

	for (unsigned int i = 0 ; i < states.size() ; ++i) {
		if (!isCharInAlphabet(states[i])) throw BadCharException(states[i], "DNA::getGeneric(const vector<string>& states): Specified base unknown.");
		vi.push_back(this->charToInt(states[i]));
	}

	return intToChar(getGeneric(vi));
}
