/*
 * DNA454.h
 *
 *  Created on: 2 nov. 2010
 *      Author: cahais
 */

#ifndef _CUSTOMNUCLEICALPHABET_H_
#define _CUSTOMNUCLEICALPHABET_H_

// Basic C++ classes
#include <string>
#include <vector>

// Bio++ classes
#include <Bpp/Seq/Alphabet/NucleicAlphabet.h>

namespace ppc {

	/**
	 * @brief This alphabet is used to deal with DNA sequences from 454 technology.
	 *
	 * It supports all 4 nucleotides (A, T, G and C) with their standard denomination.
	 * Gaps are coded by '-', unresolved characters are coded by 'X, N, O, 0 or ?'.
	 * Extensive support for generic characters (e.g. 'P', 'Y', etc.) is provided.
	 */
	class CustomNucleicAlphabet: public bpp::NucleicAlphabet {

		public:
			// Constructor
			CustomNucleicAlphabet();

			// Destructor
			virtual ~CustomNucleicAlphabet() {}

		public:
			std::vector<int> getAlias(int state) const throw (bpp::BadIntException);
			std::vector<std::string> getAlias(const std::string& state) const throw (bpp::BadCharException);
			int getGeneric(const std::vector<int>& states) const throw (bpp::BadIntException);
			std::string getGeneric(const std::vector<std::string>& states) const throw (bpp::BadCharException);
			std::string getAlphabetType() const { return "PopPhyl Custom Nucleic Alphabet (454 and Frameshift)"; }
	};

} //end of namespace PopPhyl Commons (ppc)

#endif // _CUSTOMNUCLEICALPHABET_H_
