/*
 * FrameshiftCodonAlphabet.cpp
 *
 *  Created on: 22 mai 2013
 *      Author: abernard
 */

// Includes
#include "FrameshiftCodonAlphabet.h"

// Namespaces
using namespace std;
using namespace bpp;
using namespace ppc;

/*-------------------------------*/
/*          Constructor          */
/*-------------------------------*/

FrameshiftCodonAlphabet::FrameshiftCodonAlphabet(NucleicAlphabet * alpha): StandardCodonAlphabet(alpha) {

	// Initializations
	unsigned int nbState = getNumberOfChars();

	// Create a Frameshift codon state
	AlphabetState frameshiftState(64,"!!!","FrameShift");
	AlphabetState unresolvedState(65,"NNN","Unresolved");

	// Increase the size of the alphabet
	resize(nbState+1);

	// Modify the last wo states
	setState(65,frameshiftState);
	setState(66,unresolvedState);
	remap();
}

/*--------------------------------*/
/*           Destructor           */
/*--------------------------------*/

FrameshiftCodonAlphabet::~FrameshiftCodonAlphabet() {
	// TODO Auto-generated destructor stub
}

/*--------------------------------------------------------------*/
/*           FrameshiftCodonAlphabet original methods           */
/*--------------------------------------------------------------*/

bool FrameshiftCodonAlphabet::isFrameshiftBase(const std::string& baseLetter) const {
	return baseLetter == "!";
}

bool FrameshiftCodonAlphabet::containsFrameshift(const string& codonLetters) const {

	for (size_t i = 0; i < codonLetters.size(); i++) {
		if (isFrameshiftBase(codonLetters.substr(i, 1))) return true;
	}

	return false;
}

/*----------------------------------------------------*/
/*           WordAlphabet overriden methods           */
/*----------------------------------------------------*/

int FrameshiftCodonAlphabet::charToInt(const string& codonLetters) const  throw (BadCharException) {

	// Return the int code for the frameshift ("!!!") codon if the current codon contains at least one "!" character or ask the WordAlphabet::charToInt method for all other codons
	if(containsFrameshift(codonLetters)) {
		return 64;
	} else {
		return WordAlphabet::charToInt(codonLetters);
	}
}

string FrameshiftCodonAlphabet::getName(const string& codonLetters) const  throw (BadCharException) {

	if(containsFrameshift(codonLetters)) {
		return "FrameShift";
	} else {
		return WordAlphabet::getName(codonLetters);
	}
}
