/*
 * FrameshiftCodonAlphabet.h
 *
 *  Created on: 22 mai 2013
 *      Author: abernard
 */

#ifndef FRAMESHIFTCODONALPHABET_H_
#define FRAMESHIFTCODONALPHABET_H_

// Basic C++ classes
#include <string>

// Bio++ classes
#include <Bpp/Seq/Alphabet/NucleicAlphabet.h>
#include <Bpp/Seq/Alphabet/StandardCodonAlphabet.h>
#include <Bpp/Seq/Alphabet/AlphabetExceptions.h>

namespace ppc {

	class FrameshiftCodonAlphabet: public bpp::StandardCodonAlphabet {

		public:
			// Constructor
			FrameshiftCodonAlphabet(bpp::NucleicAlphabet * alpha);

			// Destructor
			virtual ~FrameshiftCodonAlphabet();

			// FrameshiftCodonAlphabet original methods (Nucleotide level)
			bool isFrameshiftBase(const std::string& baseLetter) const;
			bool containsFrameshift(const std::string& codonLetters) const;

			// FrameshiftCodonAlphabet original methods (Codon level)
			bool isFrameshiftCodon();

			// StandardCodonAlphabet overridden methods
			std::string getAlphabetType() const { return "PopPhyl Custom Codon Alphabet (with Frameshift)"; }

			// WordAlphabet overridden methods
			int charToInt(const std::string& codonLetters) const throw (bpp::BadCharException);
			std::string getName(const std::string& state) const throw (bpp::BadCharException);
	};

} //end of namespace PopPhyl Commons (ppc)

#endif /* FRAMESHIFTCODONALPHABET_H_ */
