/*
 * MappingContainer.cpp
 *
 *  Created on: 15 déc. 2010
 *      Author: cahais
 */

#include "MappingContainer.h"

// Namespaces
using namespace std;
using namespace bpp;
using namespace ppc;

/******************************************************************************/

MappingContainer::Vertex::~Vertex()
{

}

/******************************************************************************/

MappingContainer::Vertex* MappingContainer::Vertex::getChildByNucleotide(char c) {

	for (unsigned int i = 0; i<childrens_.size(); i++) {
		if (childrens_[i]->getNucleotide() == c) return childrens_[i];
	}

	return 0;
}

/******************************************************************************/

bool MappingContainer::Vertex::isBaseOfSequence(unsigned int index) {

	for(unsigned int i = 0; i < getNumberOfSequences(); i++) {
		if (getSequenceIndex(i) == index) return true;
	}

	return false;
}

/******************************************************************************/

MappingContainer::MappingContainer(bool printReference) {

	alpha = new DNA();
	first_ = new MappingContainer::Vertex();
	first_->setNucleotide('X');
	refseq_= "";
	refseq_size_= 0;
	all_.push_back(first_);
	printReference_ = printReference;
}

/******************************************************************************/

MappingContainer::~MappingContainer() {

	for (unsigned int i = 0; i < all_.size(); i++) delete all_[i];
	delete alpha;

	map<unsigned int, sequence*>::iterator it;

	for (it = idIndex.begin(); it != idIndex.end(); it++) {
		delete it->second;
	}
}

/******************************************************************************/

MappingContainer::Vertex* MappingContainer::getNodeByRefPos(unsigned int pos, unsigned int length) {

	// If the read overstep by 5% we don't take it into consideration
	if (pos + length > refseq_size_ + (5/100) * length ) return NULL;

	return refpos_[pos];
}

/******************************************************************************/

unsigned int MappingContainer::registerSequence(string name, MappingContainer::Vertex * v) {

	//if (sequences_.find(name) != sequences_.end()) return 0;
	MappingContainer::sequence * newSequence = new MappingContainer::sequence;
	newSequence->begin = v;
	newSequence->index = sequences_.size();
	ostringstream oss;
	oss << name << "_" << newSequence->index;
	newSequence->name = oss.str();
	sequences_[newSequence->name] = *newSequence;
	idIndex[newSequence->index] = newSequence;

	return newSequence->index;
}

/******************************************************************************/

void MappingContainer::setRefSeq(string & name, string & seq) {

	if (sequences_.size() > 0) {
		cerr << "Sequence of reference already set." << endl;
		return;
	}

	unsigned int index = registerSequence(name, first_) ;
	MappingContainer::Vertex * curr = first_;

	for (unsigned int pos = 0; pos < seq.size(); pos++) {
		curr->setOrder(pos);
		refpos_[pos] = curr;

		MappingContainer::Vertex * next = addInsertion(curr, seq[pos], index);
		next->setToReference();
		next->setToNotAnInsertion();
		curr->setReference(next);
		curr = next;
	}

	ostringstream oss;
	oss << name << "_" << index;

	refseq_ = oss.str();
	refname_ = name;
	refseq_size_ = seq.size();
}

/******************************************************************************/

MappingContainer::Vertex* MappingContainer::addMatch(MappingContainer::Vertex * curr, char nuc, unsigned int index) {

	MappingContainer::Vertex * next = 0;
	next = curr->getChildByNucleotide(nuc);

	if (next) { //match case
		next->addSequence(index);
	} else { //mismatch case
		// New node creation
		next = new MappingContainer::Vertex();
		next->setNucleotide(nuc);
		next->addSequence(index);

		if (curr->getConsensus()) {
			next->setConsensus(curr->getConsensus()->getConsensus());
		} else {
			curr->setConsensus(next);
			all_.push_back(next);
			curr->addChild(next);
		}
	}

	//return next;
	return curr->getChildByIndex(0);
}

/******************************************************************************/

MappingContainer::Vertex* MappingContainer::addInsertion(MappingContainer::Vertex * curr, char nuc, unsigned int index) {

	MappingContainer::Vertex* next = 0;
	next = curr->getChildByNucleotide(nuc);
	if (next) {
		if (next->isInsertion()) {
			next->addSequence(index);
			return curr->getConsensus();
		}
	}

	// On créé un nouveau noeud
	next = new MappingContainer::Vertex();
	next->setNucleotide(nuc);
	next->addSequence(index);

	// On lui assigne les mêmes fils que son père
	//for(unsigned int i=0; i<curr->getNumberOfChildren(); i++)
	//next->addChild(curr->getChildByIndex(i));

	// Le prochain noeud de la reference du père devient le siens.
	next->setConsensus(curr->getConsensus());

	// Le nouveau noeud devient le prochain noeud de la référence pour le père.
	curr->setConsensus(next);

	// Le père perd tous ses fils et le nouveau noeud devient son unique fils.
	//curr->removeAllChildren();
	curr->addChild(next);
	next->setToInsertion();
	all_.push_back(next);

	return next;
}

/******************************************************************************/

string MappingContainer::getSequence(string name) {

	string seq = "";
	unsigned int index = getSequenceIndex(name);
	MappingContainer::Vertex* curr = getSequenceBegin(name);
	getNextNucleotide(curr, index, seq);

	return seq;
}

/******************************************************************************/

void MappingContainer::toMultiFasta(ostream & out) {

	map<unsigned int, sequence*>::iterator it;

	for (it = idIndex.begin(); it != idIndex.end(); it++) {
		string seq = "";
		out << ">" << it->second->name << endl;
		getNextNucleotide(first_, it->first, seq);
		out << seq << endl;
	}
}

/******************************************************************************/

void MappingContainer::getNextNucleotide(MappingContainer::Vertex* curr, unsigned int index, string & seq) {

	string c = "-";

	for (unsigned int i = 0; i < curr->getNumberOfChildren(); i++) {
		if (printReference_ and curr->getConsensus()->isInsertion()) c="";

		if (curr->getChildByIndex(i)->isBaseOfSequence(index)) {
			c = curr->getChildByIndex(i)->getNucleotide();
		}
	}

	seq += c;

	if (curr->getConsensus()) getNextNucleotide(curr->getConsensus(), index, seq);
}

/******************************************************************************/

void MappingContainer::printTree(MappingContainer::Vertex* curr) {

	cerr << curr->getOrder() << " " ;

	if (curr->isInsertion()) {
		cerr << "-:";
	} else {
		cerr << curr->getNucleotide() << ": ";
	}

	for (unsigned int i = 0; i < curr->getNumberOfChildren(); i++) {
		cerr << "(" <<  curr->getChildByIndex(i)->getNucleotide() << "," ;
		for (unsigned int j = 0; j < curr->getChildByIndex(i)->getNumberOfSequences(); j++) {
			cerr << curr->getChildByIndex(i)->getSequenceIndex(j) << ",";
		}
		cerr << ")";
	}
	cerr << endl;

	if (curr->getConsensus()) printTree(curr->getConsensus());
}

/******************************************************************************/

void MappingContainer::toAln(ostream & out, ostream & fasta) {

	MappingContainer::Vertex * curr = first_;
	vector<string> indiv;
	vector<string> indivBySeqID;
	indivBySequenceID(indiv, indivBySeqID);
	unsigned int nbIndiv = indiv.size();

	out << ">" << refname_ << endl;
	out << "maj\tM/P";
	for (unsigned int i = 0; i < nbIndiv; i++) out << "\t" << indiv[i];
	out << endl;

	char nuc = 'x';
	int cpt = 0;

	while(curr->getNumberOfChildren() > 0) {
		if (!curr->getReference()) break;

		//si la reference est differente de tous les reads
		if(curr->getNumberOfChildren() == 2 && curr->getReference()->getNumberOfSequences() == 1) {
			out << writeSpecialSite(curr, indiv, indivBySeqID) << endl;
		} else if (curr->getNumberOfChildren() > 1) {
			nuc = getMajority(curr);
			if (nuc != '-') out << writePolyMorphSite(curr, indiv, indivBySeqID, nuc) << endl;
		} else {
			out << writeMonoMorphSite(curr, indiv, indivBySeqID) << endl;
		}

		//if (printReference_) curr=curr->getReference();
		//else curr=curr->getConsensus();

		if (cpt%60 == 0 && cpt != 0 ) fasta << endl;
		if (nuc != '-') {fasta << curr->getReference()->getNucleotide(); cpt++;}
		nuc = 'x';

		curr = curr->getReference();
	}
}

/******************************************************************************/

string MappingContainer::writeSpecialSite(MappingContainer::Vertex* curr, vector<string> & indiv, vector<string> & indivBySeqID) {

	ostringstream line;
	map <string, unsigned int > compo;

	for (unsigned int i = 0; i < curr->getChildByIndex(1)->getNumberOfSequences(); i++) {
		string indiv = indivBySeqID[curr->getChildByIndex(1)->getSequenceIndex(i)];
		compo[indiv]++;
	}

	char nuc = curr->getChildByIndex(1)->getNucleotide();

	if (nuc == 'N') {
		line << "A" << "\tM";
		for (unsigned int i = 0; i < indiv.size(); i++) line << "\t" << 0;
	} else {
		line << nuc << "\tM";
		for (unsigned int i = 0; i < indiv.size(); i++) line << "\t" << compo[indiv[i]];
	}

	return line.str();
}

/******************************************************************************/

string MappingContainer::writeMonoMorphSite(MappingContainer::Vertex* curr, vector<string> & indiv, vector<string> & indivBySeqID) {

	ostringstream line;
	map <string, unsigned int > compo;

	for (unsigned int i = 0; i < curr->getConsensus()->getNumberOfSequences(); i++) {
		string indiv = indivBySeqID[curr->getConsensus()->getSequenceIndex(i)];
		compo[indiv]++;
	}

	char nuc = curr->getConsensus()->getNucleotide();

	if (nuc == 'N')	{
		line << "A" << "\tM";
		for (unsigned int i = 0; i < indiv.size(); i++) line << "\t" << 0;
	} else {
		line << nuc << "\tM";
		for (unsigned int i = 0; i < indiv.size(); i++) line << "\t" << compo[indiv[i]];
	}

	return line.str();
}

/******************************************************************************/

string MappingContainer::writePolyMorphSite(MappingContainer::Vertex* curr, vector<string> & indiv, vector<string> & indivBySeqID, char nuc) {

	ostringstream line;
	map <string, vector<unsigned int> > compo;

	for (unsigned int x = 0; x < indiv.size(); x++) {
		for (unsigned int y = 0; y < 4; y++) compo[indiv[x]].push_back(0);
	}

	for (unsigned int i = 0; i < curr->getNumberOfChildren(); i++) {
		for (unsigned int j = 0; j < curr->getChildByIndex(i)->getNumberOfSequences(); j++) {
			unsigned int id = curr->getChildByIndex(i)->getSequenceIndex(j);
			string ind = indivBySeqID[id];

			if (ind == "ref") continue;

			string nuc(1, curr->getChildByIndex(i)->getNucleotide());
			unsigned int nuci = alpha->charToInt(nuc);

			if (nuci < 4) compo[ind][nuci]++;
		}
	}

	line << nuc << "\tP";

	for (unsigned int i = 0; i < indiv.size(); i++) {
		unsigned int somme = compo[indiv[i]][0] + compo[indiv[i]][1] + compo[indiv[i]][2] + compo[indiv[i]][3];
		line << "\t" << somme;
		line << "[" << compo[indiv[i]][0];
		line << "/" << compo[indiv[i]][1];
		line << "/" << compo[indiv[i]][2];
		line << "/" << compo[indiv[i]][3];
		line << "]";
	}

	return line.str();
}

/******************************************************************************/

void MappingContainer::indivBySequenceID(vector<string> & indiv, vector<string> & indivBySeqID) {

	map<string, sequence>::iterator it;
	set<string> tmpSet;

	for (unsigned int i = 0; i < sequences_.size(); i++) indivBySeqID.push_back("");

	for (it = sequences_.begin(); it != sequences_.end(); it++) {
		if (it->first == refseq_) {
			indivBySeqID[it->second.index] = "ref";
		} else {
			StringTokenizer st1(it->first, "|", true, false);
			string id = st1.getToken(0) + "|" + st1.getToken(1);
			indivBySeqID[it->second.index] = id;
			tmpSet.insert(id);
		}
	}

	copy(tmpSet.begin(), tmpSet.end(), back_inserter(indiv));
}

/******************************************************************************/

MappingContainer::Vertex* MappingContainer::getVertexByPosition(unsigned int pos) {

	MappingContainer::Vertex* curr = first_;

	while(curr->getOrder() < pos) curr = curr->getReference();

	return curr;
}

/******************************************************************************/

char MappingContainer::getMajority(MappingContainer::Vertex* curr) {

	unsigned int max = 0;
	unsigned int major = 0;

	for(unsigned int i = 0; i < curr->getNumberOfChildren(); i++) {
		if (curr->getChildByIndex(i)->getNumberOfSequences() > max and curr->getChildByIndex(i)->getNucleotide() != 'N') {
			max = curr->getChildByIndex(i)->getNumberOfSequences();
			major = i;
		}
	}

	return curr->getChildByIndex(major)->getNucleotide();
}

/******************************************************************************/

void  MappingContainer::printIdIndex() {

	cerr << "idIndex"  << idIndex.size() << endl;

	map<unsigned int, sequence*>::iterator xt;

	for (xt=idIndex.begin(); xt != idIndex.end(); xt++)	cerr << xt->first << " " << xt->second->name << endl;
}
