#ifndef _QUAL_H_
#define _QUAL_H_

#include <iostream>
#include <fstream>
#include <map>

#include <Bpp/Exceptions.h>
#include <Bpp/Seq/SequenceWithQuality.h>

#include <Bpp/Text/TextTools.h>
#include <Bpp/Io/FileTools.h>
#include <Bpp/Text/StringTokenizer.h>


using namespace bpp;

namespace ppc
{

/**
 * @brief The qual sequence file format.
 *
 * Read and write from/to Qual files.
 */
class Qual
{

public:
  
  /**
   * @brief The maximum number of chars to be written on a line.
   */
  unsigned int charsByLine_;

public:

  /**
   * @brief Build a new Qual object.
   *
   * @param charsByLine Number of character per line when writing files.
   */
  Qual(unsigned int charsByLine = 100): charsByLine_(charsByLine) {}

  // Class destructor
  virtual ~Qual() {}

public:
    
  const std::string getFormatName() const { return "QUAL file"; };
  const std::string getFormatDescription() const
  {
    return "Sequence name (preceded by >) in one line, sequence content, no comments";
  }

  void nextSequence(std::istream& input, std::vector<int> & qualities, std::string & name) const throw (Exception);
  void nextSequence(std::istream& input, SequenceWithQuality & swq) const throw (Exception)
  {
	std::string name="";
	std::vector<int> qualities;
    nextSequence(input, qualities, name);
    swq.setQualities(qualities);
  }

  void writeSequence(std::ostream& output, std::vector<int> qualities, std::string name) const throw (Exception);
  void writeSequence(std::ostream& output, SequenceWithQuality & swq) const throw (Exception)
  {
	  writeSequence(output, swq.getQualities(), swq.getName());
  }

};

} //end of namespace bpp.

#endif // _QUAL_H_

