/*
 * Sam.cpp
 *
 *  Created on: 17 oct. 2011
 *      Author: cahais
 */

#include "Sam.h"

using namespace std;
using namespace bpp;
using namespace ppc;

Sam::Sam()
{
	// TODO Auto-generated constructor stub
	alpha = new CustomNucleicAlphabet();
}

Sam::~Sam()
{
	// TODO Auto-generated destructor stub
	delete alpha;
}


int Sam::readSamLine(std::ifstream & input, Sam::Samline & samline)
{
  std::string lineBuffer="";
  getline(input, lineBuffer);
  if (lineBuffer[0] == '@' or lineBuffer== "") return 0;
  StringTokenizer * st  = new StringTokenizer(lineBuffer,"\t",true,false);
  if (st->numberOfRemainingTokens() < 10) return 0;

  samline.qname = st->nextToken();
  samline.flag = atoi(st->nextToken().c_str());
  samline.rname = st->nextToken();
  samline.pos = atoi(st->nextToken().c_str());
  samline.mapq = atoi(st->nextToken().c_str());
  samline.cigar = st->nextToken();
  samline.mrnm = st->nextToken();
  samline.mpos = atoi(st->nextToken().c_str());
  samline.isize = atoi(st->nextToken().c_str());
  samline.seq = st->nextToken();
  samline.qual = st->nextToken();
  //if (st->hasMoreToken()) samline->opt = st->getToken(11);
  delete st;
  return 1;
}

/* @name  cigar2String
 * @brief convert samline into a string
 * @param a Samline,
 * @return a String
 */
string Sam::samline2StringSequence(Samline & saml)
{
  string chiffre;
  string sequence="";
  size_t pos = 0; // position sur le read

  for (size_t i=0; i< saml.cigar.size(); i++)
  {
    if ((int)saml.cigar[i] > 47 and (int)saml.cigar[i] < 58)
    {
      chiffre+=saml.cigar[i];
    }
    else
    {
    	size_t value = atoi(chiffre.c_str());
    	switch (saml.cigar[i])
    	{
    		case 'M':                                   // cas d'un match
    	        for(size_t j=0; j< value; j++){ sequence+=saml.seq[pos]; pos++; }
    	        break;

    		case 'D': 								   // cas d'une deletion
    			for(size_t j=0; j< value; j++) sequence+='-';
    			break;

    		case 'I':                                  // cas d'une insertion
    			for(size_t j=0; j< value; j++) pos++;
    			break;

    		case 'S': 						           // cas d'une substitution
    			for(size_t j=0; j< value; j++){ sequence+=saml.seq[pos]; pos++; }
    			break;

    		default :
    			cerr << "wrong cigar string : " << saml.cigar << endl;
    			break;
    	}
    	chiffre = "";
    }
  }
  return sequence;
}

/* @name  samline2sequence
 * @brief convert samline into a sequence
 * @param a Samline,
 * @return a BasicSequence
 */
BasicSequence* Sam::samline2sequence(Sam::Samline & saml)
{
	string chiffre = "";
	BasicSequence* bs = new BasicSequence(saml.qname, saml.seq, alpha);

	for (size_t i=0; i< saml.cigar.size(); i++)
	{
		if ((int)saml.cigar[i] > 47 and (int)saml.cigar[i] < 58)
		{
			chiffre+=saml.cigar[i];
		}
		else
		{
			size_t pos = atoi(chiffre.c_str());
			switch (saml.cigar[i])
			{
				case 'M': break;                            // cas d'un match
			    case 'D': bs->addElement(pos, "-"); break;  // cas d'une deletion
			    case 'I': bs->deleteElement(pos);   break;  // cas d'une insertion
			    case 'S': break;							// cas d'une substitution
			    default : cerr << "wrong cigar string : " << saml.cigar << endl; break;
		    }
			chiffre = "";
		}
	}
	return bs;
}


void Sam::build(std::ifstream & input) throw (Exception)
{

  std::string lineBuffer="";
  while(!input.eof())
  {
	  ios::pos_type pos = input.tellg();
	  getline(input, lineBuffer);
	  if (lineBuffer[0] == '@'){ continue; }
	  StringTokenizer st(lineBuffer,"\t",true,false);
	  if (st.numberOfRemainingTokens() > 1 and st.getToken(2) != "*")
	    index_[st.getToken(2)].push_back(pos);
  }
  input.clear();
}


