/*
 * ParametersManagement.h
 *
 *  Created on: 26 sept. 2012
 *      Author: abernard
 */

#ifndef PARAMETERSMANAGEMENT_H_
#define PARAMETERSMANAGEMENT_H_

// Basic C++ classes
#include <string>

class ParametersManagement {

	protected:
		// Command line parameters related attributes
		std::string _alignement_file;
		std::string _ingroup_species_name;
		std::string _outgroup_species_name;
		std::string _execution_mode;
		int test;
		unsigned int _min_gapN_site;
		double _max_gapN_seq;
		unsigned int _tolerance_zone;
		unsigned int _DFEa_write_start;
		unsigned int _DFEa_min_NS_SNP;

		unsigned int _min_nb_codon;
		unsigned int _sample_size;
		double _kappa;

		bool _remove_frameshift;
		bool _allow_internal_bc;
		bool _DEBUG;

		// Default values
		std::string Default_execution_mode;

		unsigned int Default_min_gapN_site;
		double Default_max_gapN_seq;
		unsigned int Default_tolerance_zone;
		unsigned int Default_DFEa_write_start;
		unsigned int Default_DFEa_min_NS_SNP;

		unsigned int Default_min_nb_codon;
		unsigned int Default_sample_size;
		double Default_kappa;

		bool Default_remove_frameshift;
		bool Default_allow_internal_bc;
		bool Default_DEBUG;

		// Version related attributes
		double _versionNumber;
		std::string _versionDate;

	public:
		// Constructor and destructor
		ParametersManagement(const double versionNumber, const std::string versionDate, int nbArguments, char **allArguments);
		virtual ~ParametersManagement();

		// Accessors

		// GET
		std::string get_alignement_file();
		std::string get_ingroup_species_name();
		std::string get_outgroup_species_name();
		std::string get_execution_mode();

		unsigned int get_min_gapN_site();
		double get_max_gapN_seq();
		unsigned int get_tolerance_zone();
		unsigned int get_DFEa_write_start();
		unsigned int get_DFEa_min_NS_SNP();

		unsigned int get_min_nb_codon();
		unsigned int get_sample_size();
		double get_kappa();

		bool get_remove_frameshift();
		bool get_allow_internal_bc();

		bool get_DEBUG();

		// SET
		void set_sample_size(unsigned int validSampleSize);

		// Other methods
		void checkParameters();
		void displayHelp();
};

#endif /* PARAMETERSMANAGEMENT_H_ */
