/*
 * ParametersManagement.h
 *
 *  Created on: 23 oct. 2012
 *      Author: abernard
 */

#ifndef PARAMETERSMANAGEMENT_H_
#define PARAMETERSMANAGEMENT_H_

// Basic C++ classes
#include <string>

class ParametersManagement {
	protected:
		// Command line parameters related attributes
		std::string _alignement_file;
//		std::string _tree_file;
		std::string _population_file;

		std::string _ingroup_species_name;
		std::string _outgroup_species_name;

		std::string _execution_mode;
		std::string _output_file_name;
		std::string _contig_usage;

		unsigned int _nb_gc_bin;
		unsigned int _min_gapN_site;
		double _max_gapN_seq;
		unsigned int _tolerance_zone;
		unsigned int _min_nb_codon;
		double _kappa;

		unsigned int _bootstrap_replicates;
		unsigned int _bootstrap_sort_precision;
		double _bootstrap_confidence;

		bool _remove_frameshift;
		bool _compute_distances;
		bool _allow_internal_bc;
		bool _print_header;

		bool _DEBUG;

		// Default values
		std::string Default_execution_mode;
		std::string Default_contig_usage;

		unsigned int Default_nb_gc_bin;
		unsigned int Default_min_gapN_site;
		double Default_max_gapN_seq;
		unsigned int Default_tolerance_zone;
		unsigned int Default_min_nb_codon;
		double Default_kappa;

		unsigned int Default_bootstrap_replicates;
		unsigned int Default_bootstrap_sort_precision;
		double Default_bootstrap_confidence;

		bool Default_remove_frameshift;
		bool Default_compute_distances;
		bool Default_allow_internal_bc;
		bool Default_print_header;

		bool Default_DEBUG;

		// Version related attributes
		double _versionNumber;
		std::string _versionDate;

	public:
		// Constructor and destructor
		ParametersManagement(const double versionNumber, const std::string versionDate, int nbArguments, char **allArguments);
		virtual ~ParametersManagement();

		// Accessors - Get
		std::string get_alignement_file();
//		std::string get_tree_file();
		std::string get_population_file();

		std::string get_ingroup_species_name();
		std::string get_outgroup_species_name();

		std::string get_execution_mode();
		std::string get_output_file_name();
		std::string get_contig_usage();

		unsigned int get_nb_gc_bin();
		unsigned int get_min_gapN_site();
		double get_max_gapN_seq();
		unsigned int get_tolerance_zone();
		unsigned int get_min_nb_codon();
		double get_kappa();

		unsigned int get_bootstrap_replicates();
		unsigned int get_bootstrap_sort_precision();
		double get_bootstrap_confidence();

		bool get_remove_frameshift();
		bool get_compute_distances();
		bool get_allow_internal_bc();
		bool get_print_header();

		bool get_DEBUG();

		// Accessors - Set
		void set_outgroup_species_name(std::string speciesName);

		// Other methods
		void checkParameters();
		void displayHelp();
};

#endif /* PARAMETERSMANAGEMENT_H_ */
