/*
 * ResultsWarehouse.h
 *
 *  Created on: 25 oct. 2012
 *      Author: abernard
 */

#ifndef RESULTSWAREHOUSE_H_
#define RESULTSWAREHOUSE_H_

// Basic C++ classes
#include <string>
#include <vector>
#include <map>

// PopPhyl Project Developments
#include <ppc/PopGen/Population.h>
#include <ppc/PopGen/DistanceBetweenIndividuals.h>

#include "SpeciesDataWarehouse.h"
#include "ParametersManagement.h"

class ResultsWarehouse {

	protected:
		std::string _name;
		std::string _status;

		unsigned int _nb_bi_allelic_SNP;
		unsigned int _nb_tri_allelic_SNP;
		unsigned int _nb_quadri_allelic_SNP;

		unsigned int _nb_sequences_in_consensus;
		unsigned int _nb_sites_in_consensus;

		double _nN;
		double _nS;

		int _fixN;
		int _fixS;
		int _polymN_0;
		int _polymS_0;
		int _polymN_02;
		int _polymS_02;
		int _MK_nN;
		int _MK_nS;

		std::vector<double> _GC_allele_frequency_classes;
		std::vector< std::vector<double> > _Fst;

		std::map <std::string, double> _individual_heterozygosity;

		std::map <std::string, ppc::DistanceBetweenIndividuals*> _distances;

		std::vector<SpeciesDataWarehouse*> _allSpeciesData;

	public:
		// Constructor
		ResultsWarehouse(ppc::Population* popData, unsigned int numberOfBin);

		// Destructor
		virtual ~ResultsWarehouse();

		// Debug method
		static void warehouseInventory (ResultsWarehouse* warehouse, ParametersManagement* params, ppc::Population* popData, std::vector<double>& alleleFrequencyClass);

		// Accessors - Set
		void set_name(std::string contigName);
		void set_status(std::string newStatus);

		void set_nb_bi_allelic_SNP(unsigned int newValue);
		void set_nb_tri_allelic_SNP(unsigned int newValue);
		void set_nb_quadri_allelic_SNP(unsigned int newValue);

		void set_nb_sequences_in_consensus(unsigned int newValue);
		void set_nb_sites_in_consensus(unsigned int newValue);

		void set_nN(double newValue);
		void set_nS(double newValue);

		void set_fixN(int newValue);
		void set_fixS(int newValue);
		void set_polymN_0(int newValue);
		void set_polymS_0(int newValue);
		void set_polymN_02(int newValue);
		void set_polymS_02(int newValue);
		void set_MK_nN(int newValue);
		void set_MK_nS(int newValue);

		void increase_class(int classNumber, int increaseBy);

		void set_specific_Fst(unsigned int firstPopulationId, unsigned int secondPopulationId, double computedFst);
		void set_specific_individual_heterozygosity(std::string individualName, double computedValue);
		void set_specific_distance_object(std::string coupleName, ppc::DistanceBetweenIndividuals* pointerToDistanceObject);

		void store_species_data(SpeciesDataWarehouse* newSpeciesData);

		// Accessors - Get
		std::string get_name();
		std::string get_status();

		unsigned int get_nb_bi_allelic_SNP();
		unsigned int get_nb_tri_allelic_SNP();
		unsigned int get_nb_quadri_allelic_SNP();

		unsigned int get_nb_sequences_in_consensus();
		unsigned int get_nb_sites_in_consensus();

		double get_nN();
		double get_nS();

		double get_specific_Fst(unsigned int firstPopulationId, unsigned int secondPopulationId);
		double get_specific_individual_heterozygosity(std::string individualName);
		ppc::DistanceBetweenIndividuals* get_specific_distance_object(std::string coupleName);

		int get_fixN();
		int get_fixS();
		int get_polymN_0();
		int get_polymS_0();
		int get_polymN_02();
		int get_polymS_02();
		int get_MK_nN();
		int get_MK_nS();

		std::vector<double> get_GC_allele_frequency_classes();

		std::vector< std::vector<double> > get_all_Fst();
		std::map <std::string, double> get_all_individual_heterozygosity();
		std::map <std::string, ppc::DistanceBetweenIndividuals*> get_all_distance_objects();

		std::vector<SpeciesDataWarehouse*> get_all_species_data();
		SpeciesDataWarehouse* get_specific_species_data(std::string speciesName);
};

#endif /* RESULTSWAREHOUSE_H_ */
