/*
 * geno2pfas.hpp
 *
 *  Created on: 15 avr. 2013
 *      Author: vcahais
 */

#ifndef GENO2PFAS_HPP_
#define GENO2PFAS_HPP_

#include <Bpp/Seq/Sequence.h>
#include <Bpp/Seq/Alphabet/DNA.h>
#include <Bpp/Seq/Io/Fasta.h>

using namespace std;
using namespace bpp;

namespace bpp
{

class Geno2pfas
{

public:
	double threshold_;

public :

	Geno2pfas(): threshold_(0.95){}
	~Geno2pfas(){}

	/**************************************************************************************/
	/*geno2pfas::write_alleles
	* @INPUT : AlrContig object, output stream
	* @DESC  : write main output (fasta file)
	*/
	/**************************************************************************************/
	void gen2seq(string sctg, vector<Sequence*> & sequences)
	{
		Sequence * seq=0;
		Alphabet * alpha = new DNA();
		string linebuffer="";
		istringstream input(sctg);

		//get contig name
		getline(input, linebuffer);
		string contigname=linebuffer.substr(1,linebuffer.size()-1);

		//create alleles sequences
		getline(input, linebuffer);
		StringTokenizer st1 (linebuffer, "\t", false, false);
		st1.nextToken();
		while(st1.hasMoreToken())
		{
			string indname = st1.nextToken();
			for (unsigned int i=1; i<3 ;i++)
			{
				ostringstream allname;
				allname << contigname<< "|" << indname << "|Allele_" << i;
				seq = new BasicSequence(allname.str(),"",alpha);
				sequences.push_back(seq);
			}
		}
		//fill alleles sequences
		while (!input.eof())
		{
			getline(input, linebuffer);
			if (linebuffer == "") continue;
			StringTokenizer st2 (linebuffer, "\t|", false, false);
			unsigned int pos = atoi(st2.nextToken().c_str());
			for (unsigned int i=0; st2.hasMoreToken(); i+=2)
			{
				string site = st2.nextToken();
				double prob = atof(st2.nextToken().c_str());
				if (prob > threshold_)
				{
					sequences[i]->addElement(utils::toint(site[0]));
					sequences[i+1]->addElement(utils::toint(site[1]));
				}
				else
				{
					sequences[i]->addElement(14);
					sequences[i+1]->addElement(14);
				}
			}
		}

	}

	void write_alleles(vector<Sequence*> & sequences,ofstream & out)
	{
		//write sequences into output file
		Fasta writer(60);
		for (unsigned int i=0; i<sequences.size(); i++)
			writer.writeSequence(out, *sequences[i]);
	}

};

}



#endif /* GENO2PFAS_HPP_ */
