//============================================================================
// Name        : Logvrais0.hpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef LOGVRAIS0_H_
#define LOGVRAIS0_H_

#include <valarray>
#include "../Stat/Stat.h"

#include <Bpp/Numeric/Function/Functions.h>

using namespace std;
using namespace bpp;

namespace bpp
{

class Logvrais0 : public virtual Function, public AbstractParametrizable
{
	private:
		double LL;
		Stat * s;
		double prec;

	public:
		Logvrais0(Stat* stat,double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			addParameter_(Parameter("xi", prec, new IncludingInterval(prec, 1-prec, prec)));
		}

		Logvrais0* clone() const { return new Logvrais0(*this); }

	public:

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }

		void fireParameterChanged(const ParameterList& pl) {

			double x = getParameterValue("xi");
			LL = s->logvrais0(x);
		}
};


} /* namespace bpp */
#endif /* LOGVRAIS0_H_ */
