//============================================================================
// Name        : Logvrais0_dir.hpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef Logvrais0_dir_H_
#define Logvrais0_dir_H_

#include <valarray>
#include "../Stat/Stat.h"
#include <limits>

#include <Bpp/Numeric/Function/Functions.h>
//#include <Bpp/Numeric/Parameter.h>
//#include <Bpp/Numeric/Function/ThreePointsNumericalDerivative.h>
//#include <Bpp/Numeric/Constraints.h>
//#include <limits>

using namespace std;
using namespace bpp;

namespace bpp
{

class Logvrais0_dir : public virtual Function, public AbstractParametrizable
{
	private:
		double LL;
		Stat * s;
		double prec;

	public:
		Logvrais0_dir(Stat * stat, double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			//addParameter_(new Parameter("xi", prec, new IntervalConstraint(prec, 1-prec, 1, 1, prec)));
			//addParameter_(new Parameter("alpha", 1-prec, new IntervalConstraint(1-prec, numeric_limits<double>::infinity(), 1, 1, prec)));
			addParameter_(Parameter("xi", prec, new IncludingInterval(prec, 1-prec, prec)));
			//addParameter_(Parameter("alpha", 100, new IncludingInterval(1, NumConstants::VERY_BIG, prec)));
			addParameter_(Parameter("alpha", 100, new IncludingPositiveReal(1,prec)));
		}

		Logvrais0_dir* clone() const { return new Logvrais0_dir(*this); }

	public:

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }


		void fireParameterChanged(const ParameterList& pl) {

			double x = getParameterValue("xi");
			double alpha = getParameterValue("alpha");
			LL = s->logvrais0_dir(x,alpha);
		}
};

/*
class Logvrais0der_dir : public virtual DerivableFirstOrder, public AbstractParametrizable
{
	private:
		double LL;
		stat * s;
		double prec;
		bool isFirstOrderDerivatives;
		Logvrais0_dir * log0;
		ThreePointsNumericalDerivative * tpnd;

	public:
		Logvrais0der_dir(stat * stat, double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			addParameter_(new Parameter("xi", prec, new IntervalConstraint(0.0, 1.0, 0, 0, prec)));
			addParameter_(new Parameter("alpha", prec, new IntervalConstraint(0.0, 1.0, 0, 0, prec)));
			log0 = new Logvrais0_dir(s,prec);
			tpnd = new ThreePointsNumericalDerivative(log0);
			tpnd->setParametersToDerivate(log0->getParameters().getParameterNames());
		}

		Logvrais0der_dir* clone() const { return new Logvrais0der_dir(*this); }

	public:

		void enableFirstOrderDerivatives(bool yn)
		{
			isFirstOrderDerivatives = true;
		}

		bool enableFirstOrderDerivatives() const
		{
			return isFirstOrderDerivatives;
		}

		double getFirstOrderDerivative(const std::string& variable) const throw (Exception)
		{
			tpnd->setParameters(getParameters());
			return tpnd->getFirstOrderDerivative(variable);
		}

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }

		void fireParameterChanged(const ParameterList& pl) {

			double x = getParameterValue("xi");
			double alpha = getParameterValue("alpha");
			LL = s->logvrais0_dir(x,alpha);
		}
};
*/


} /* namespace bpp */
#endif /* Logvrais0_dir_H_ */
