//============================================================================
// Name        : Logvrais1.hpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef Logvrais1_H_
#define Logvrais1_H_

#include <valarray>
#include "../Stat/Stat.h"

#include <Bpp/Numeric/Function/Functions.h>

using namespace std;
using namespace bpp;

namespace bpp
{

class Logvrais1 : public virtual Function, public AbstractParametrizable
{
	private:
		double LL;
		Stat * s;
		double prec;

	public:
		Logvrais1(Stat * stat, double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			addParameter_(Parameter("p1", prec, new IncludingInterval(prec, 1-prec, prec)));
			addParameter_(Parameter("p2", 1-prec, new IncludingInterval(prec, 1-prec, prec)));
			addParameter_(Parameter("xi", 0.5, new IncludingInterval(0.5, 1-prec, prec)));
		}

		Logvrais1* clone() const { return new Logvrais1(*this); }

	public:

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }

		void fireParameterChanged(const ParameterList& pl) {

			double p1 = getParameterValue("p1");
			double p2 = getParameterValue("p2");
			double x = getParameterValue("xi");

			LL = s->logvrais1(p1,p2,x);
		}

};

} /* namespace bpp */
#endif /* Logvrais1_H_ */
