//============================================================================
// Name        : Logvrais1_dir.hpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef Logvrais1_dir_H_
#define Logvrais1_dir_H_

#include <valarray>
#include "../Stat/Stat.h"

#include <Bpp/Numeric/Function/Functions.h>
#include <Bpp/Numeric/NumConstants.h>
//#include <Bpp/Numeric/Parameter.h>

using namespace std;
using namespace bpp;

namespace bpp
{

class Logvrais1_dir : public virtual Function, public AbstractParametrizable
{
	private:
		double LL;
		Stat * s;
		double prec;

	public:
		Logvrais1_dir(Stat * stat, double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			//addParameter_(new Parameter("p1", 1-prec, new IntervalConstraint(prec, 1-prec, 1, 1, prec)));
			//addParameter_(new Parameter("p2", prec, new IntervalConstraint(prec, 1-prec, 1, 1, prec)));
			//addParameter_(new Parameter("xi",  0.5, new IntervalConstraint(0.5, 1-prec, 1, 1, prec)));
			//addParameter_(new Parameter("alpha", 100, new IntervalConstraint(1, numeric_limits<double>::infinity(), 1, 1, prec)));

			addParameter_(Parameter("p1", 1-prec, new IncludingInterval(prec, 1-prec, prec)));
			addParameter_(Parameter("p2", prec, new IncludingInterval(prec, 1-prec, prec)));
			addParameter_(Parameter("xi", 0.5, new IncludingInterval(0.5, 1-prec, prec)));
			addParameter_(Parameter("alpha", 100, new IncludingPositiveReal(1,prec)));

		}

		Logvrais1_dir* clone() const { return new Logvrais1_dir(*this); }

	public:

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }

		void fireParameterChanged(const ParameterList& pl) {

			float p1 = getParameterValue("p1");
			float p2 = getParameterValue("p2");
			float x = getParameterValue("xi");
			float alpha = getParameterValue("alpha");

			LL = s->logvrais1_dir(p1,p2,x,alpha);
		}

};


/*
class Logvrais1der_dir : public virtual DerivableFirstOrder, public AbstractParametrizable
{
	private:
		double LL;
		stat * s;
		double prec;
		bool isFirstOrderDerivatives;
		Logvrais1_dir * log1;
		ThreePointsNumericalDerivative * tpnd;

	public:
		Logvrais1der_dir(stat * stat, double p) : AbstractParametrizable(""), LL(0), s(stat), prec(p)
		{
			addParameter_(new Parameter("p1", prec, new IntervalConstraint(0.0, 1.0, 0, 0, prec)));
			addParameter_(new Parameter("p2", prec, new IntervalConstraint(0.0, 1.0, 0, 0, prec)));
			addParameter_(new Parameter("xi",  0.5, new IntervalConstraint(0.5-prec, 1.0, 0, 0, prec)));
			addParameter_(new Parameter("alpha", prec, new IntervalConstraint(0.0, 1.0, 0, 0, prec )));
			log1 = new Logvrais1_dir(s,prec);
			tpnd = new ThreePointsNumericalDerivative(log1);
			tpnd->setParametersToDerivate(log1->getParameters().getParameterNames());
		}

		Logvrais1der_dir* clone() const { return new Logvrais1der_dir(*this); }

	public:

		void enableFirstOrderDerivatives(bool yn)
		{
			isFirstOrderDerivatives = true;
		}

		bool enableFirstOrderDerivatives() const
		{
			return isFirstOrderDerivatives;
		}

		double getFirstOrderDerivative(const std::string& variable) const throw (Exception)
		{

			tpnd->setParameters(getParameters());
			return tpnd->getFirstOrderDerivative(variable);
		}

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return LL; }

		void fireParameterChanged(const ParameterList& pl) {

			float p1 = getParameterValue("p1");
			float p2 = getParameterValue("p2");
			float x = getParameterValue("xi");
			float alpha = getParameterValue("alpha");
			LL = s->logvrais1_dir(p1,p2,x,alpha);
		}

};
*/

} /* namespace bpp */
#endif /* Logvrais1_dir_H_ */
