//============================================================================
// Name        : Paraclean.cpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#include "Paraclean.h"

namespace bpp
{

Paraclean::Paraclean() : fis_(0.0), threshold_(0.001), error_ (0.0), prec_(0.001),
		polypara_ (0.0), dirichlet_ (0), optimizer_("bfgs")
{
	// TODO Auto-generated constructor stub

}

Paraclean::~Paraclean()
{
	// TODO Auto-generated destructor stub
}

} /* namespace bpp */

double Paraclean::optimization0(Optimizer & optim, ParameterList & pl ,double xi)
{
	pl.setParameterValue("xi",xi);
	optim.init(pl);
	return optim.optimize();
}

double Paraclean::optimization0dir(Optimizer & optim, ParameterList & pl , double xi, double alpha)
{
 	pl.setParameterValue("xi",xi);
	pl.setParameterValue("alpha",alpha);
	optim.init(pl);
	return optim.optimize();
}

double Paraclean::optimization1(Optimizer & optim, ParameterList & pl, double p1, double p2, double xi)
{
	pl.setParameterValue("p1",p1);
	pl.setParameterValue("p2",p2);
	pl.setParameterValue("xi",xi);
	optim.init(pl);
	return optim.optimize();
}

double Paraclean::optimization1dir(Optimizer & optim, ParameterList & pl, double p1, double p2, double xi, double alpha)
{
	pl.setParameterValue("p1",p1);
	pl.setParameterValue("p2",p2);
	pl.setParameterValue("xi",xi);
	pl.setParameterValue("alpha",alpha);
	optim.init(pl);
	return optim.optimize();
}

double Paraclean::assignation(valarray<unsigned int> & geno, Stat & mystat)
{
	double dist=0;
	unsigned int nbindiv_ = geno.size()/4;
	//Stat mystat(geno, error_, fis_, prec_);
	mystat.set_values(geno);
	mystat.set_error(error_);
	mystat.set_fis(fis_);
	utils::sortMatrixByColumnSum(mystat.values_);
	//utils::printMatrix(mystat.values_);

	//starting values for optimization
	valarray<unsigned int> column = mystat.values_[slice(0,nbindiv_,4)];
	unsigned int n1 = column.sum();
	column =  mystat.values_[slice(1,nbindiv_,4)];
	unsigned int n2 = column.sum();
	double xi = (double)n1/(n1+n2);

	if (dirichlet_)
	{
		//Optimization of model 0.dir
		Logvrais0_dir log0dir(&mystat,prec_);
		//ParameterList pl0 = log0dir.getParameters();

		ReparametrizationFunctionWrapper rpf0(&log0dir,false);
		ParameterList pl0 = rpf0.getParameters();

		ThreePointsNumericalDerivative tpnd0(&rpf0);
		//ThreePointsNumericalDerivative tpnd0(&log0dir);
		tpnd0.setParametersToDerivate(pl0.getParameterNames());

		//BfgsMultiDimensions optim0(&log0dir);
		AbstractOptimizer * optim0 = 0;
		if (optimizer_ == "bfgs") optim0 = new BfgsMultiDimensions(&tpnd0);
		if (optimizer_ == "newton") optim0 = new PseudoNewtonOptimizer(&tpnd0);
		optim0->setVerbose(0);
		optim0->setProfiler(0);
		optim0->setMessageHandler(0);
		optim0->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);

		double ml0dir=0;
		try{ml0dir   = optimization0dir(*optim0, pl0, xi, 100);}
		catch (Exception & e){return -1;}
		delete optim0;

		double ml0dir_p = log0dir.getParameterValue("xi");
		double ml0dir_a = log0dir.getParameterValue("alpha");

		//Optimization of model1
		Logvrais1_dir log1dir(&mystat,prec_);

		ReparametrizationFunctionWrapper rpf1(&log1dir,false);
		ParameterList pl1 = rpf1.getParameters();

		ThreePointsNumericalDerivative tpnd1(&rpf1);
		tpnd1.setParametersToDerivate(pl1.getParameterNames());

		AbstractOptimizer * optim1 = 0;
		if (optimizer_ == "bfgs") optim1 = new BfgsMultiDimensions(&tpnd1);
		if (optimizer_ == "newton") optim1 = new PseudoNewtonOptimizer(&tpnd1);
		optim1->setVerbose(0);
		optim1->setProfiler(0);
		optim1->setMessageHandler(0);
		optim1->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
		double	ml1dira=0;
		try{ml1dira = optimization1dir(*optim1, pl1, 1-prec_, prec_, xi, 100);}
		catch (Exception & e){return -1;}

		//Check of the convergence: the likelihood must be
		//lower or equal than logvrais(p_estim0,p_estim0,1-prec)
		//where p_estim0 is the frequency estimated in model 0
		//This comparison is done instead of the direct comparison with the likelihood of model 0
		//because of the boundaries that are not 0 and 1
		//double maxLL = maxLL1dir(log1dir, pl, ml0dir_p, ml0dir_p, 1-prec_, ml0dir_a);
		double maxLL = mystat.logvrais1_dir(ml0dir_p, ml0dir_p, 1-prec_, ml0dir_a);

		double ml1dir  = ml1dira;
		double ml1dirb = 0;
		double ml1dirc = 0;
		if (ml1dira > maxLL)
		{
			try{ml1dirb = optimization1dir(*optim1, pl1, prec_, 1-prec_, xi, 100);}
			catch (Exception & e){ml1dirb=ml1dira;}
			if (ml1dirb < maxLL)	{ml1dir = ml1dirb;}
			else
			{
				try{ml1dirc = optimization1dir(*optim1, pl1, xi, xi, 0.5, 100);}
				catch (Exception & e){ml1dirc=ml1dira;}
				if (ml1dirc < maxLL)	{ml1dir = ml1dirc;}
				else
				{
					if(ml1dirb < ml1dira) ml1dir = ml1dirb;
					if(ml1dirc < ml1dirb) ml1dir = ml1dirc;
				}
			}
		}
		delete optim1;

		double q = 2*(ml0dir-ml1dir);
		if (q<0) q=0;
		dist = gsl_cdf_chisq_Q(q,2);

		/*cerr << "ml0dir=" << ml0dir;
		cerr << " ml1dira=" << ml1dira;
		cerr << " ml1dirb=" << ml1dirb;
		cerr << " ml1dirc=" << ml1dirc;
		cerr << " q=" << q << " dist=" << dist;
		cerr << endl;*/

	}
	else
	{
		//Optimization of model 0
		Logvrais0 log0(&mystat,prec_);

		ReparametrizationFunctionWrapper rpf0(&log0,false);
		ParameterList pl0 = rpf0.getParameters();

		ThreePointsNumericalDerivative tpnd0(&rpf0);
		tpnd0.setParametersToDerivate(pl0.getParameterNames());

		AbstractOptimizer * optim0 = 0;
		if (optimizer_ == "bfgs") optim0 = new BfgsMultiDimensions(&tpnd0);
		if (optimizer_ == "newton") optim0 = new PseudoNewtonOptimizer(&tpnd0);
		optim0->setVerbose(0);
		optim0->setProfiler(0);
		optim0->setMessageHandler(0);
		optim0->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);

		double ml0=0;
		try{double ml0 = optimization0(*optim0, pl0, xi);}
		catch (Exception & e){return -1;}
		delete optim0;

		double ml0_p = log0.getParameterValue("xi");

	    //Optimization of model1
		Logvrais1 log1(&mystat,prec_);

		ReparametrizationFunctionWrapper rpf1(&log1,false);
		ParameterList pl1 = rpf1.getParameters();

		ThreePointsNumericalDerivative tpnd1(&rpf1);
		tpnd1.setParametersToDerivate(pl1.getParameterNames());

		AbstractOptimizer * optim1 = 0;
		if (optimizer_ == "bfgs") optim1 = new BfgsMultiDimensions(&tpnd1);
		if (optimizer_ == "newton") optim1 = new PseudoNewtonOptimizer(&tpnd1);
		optim1->setVerbose(0);
		optim1->setProfiler(0);
		optim1->setMessageHandler(0);
		optim1->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
		double ml1a=0;
		try{ml1a = optimization1(*optim1, pl1, 1-prec_, prec_, xi);}
		catch (Exception & e){return -1;}

		//Check of the convergence: the likelihood must be
		//lower or equal than logvrais(p_estim0,p_estim0,1-prec)
		//where p_estim0 is the frequency estimated in model 0
		//This comparison is done instead of the direct comparison with the likelihood of model 0
		//because of the boundaries that are not 0 and 1
		//double maxLL = maxLL1(log1, pl, ml0_p, ml0_p, 1-prec_);
		double maxLL = mystat.logvrais1(ml0_p, ml0_p, 1-prec_);

		double ml1  = ml1a;
		double ml1b = 0;
		double ml1c = 0;
		if (ml1a>maxLL)
		{
			try{ml1b = optimization1(*optim1, pl1, prec_, 1-prec_, xi);}
			catch (Exception & e){ml1b=ml1a;}
			if (ml1b<maxLL)	{ml1 = ml1b;}
			else
			{
				try{ml1c = optimization1(*optim1, pl1, xi, xi, 0.5);}
				catch (Exception & e){ml1c=ml1a;}
				if (ml1c<maxLL)	{ml1 = ml1c;}
				else
				{
					if(ml1b < ml1a) ml1 = ml1b;
					if(ml1c < ml1b) ml1 = ml1c;
				}
			}
		}
		delete optim1;

		double q = 2*(ml0-ml1);
		if (q<0) q=0;
		dist = gsl_cdf_chisq_Q(q,2);

	}

	return dist;

}
