//============================================================================
// Name        : Paraclean.h
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 12 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef PARACLEAN_H_
#define PARACLEAN_H_

#include <iostream>
#include <cstdlib>
#include <time.h>

#include "../Stat/Stat.h"
#include "../Utils.hpp"

#include <Bpp/Numeric/Function/Functions.h>
#include <Bpp/Numeric/Function/BfgsMultiDimensions.h>
#include <Bpp/Numeric/Function/AbstractOptimizer.h>
#include <Bpp/Numeric/Function/DownhillSimplexMethod.h>
#include <Bpp/Numeric/Function/ReparametrizationFunctionWrapper.h>
#include <Bpp/Numeric/Function/ThreePointsNumericalDerivative.h>
#include <Bpp/Phyl/Likelihood/PseudoNewtonOptimizer.h>

#include "Logvrais0.hpp"
#include "Logvrais0_dir.hpp"
#include "Logvrais1.hpp"
#include "Logvrais1_dir.hpp"

#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>

using namespace std;

namespace bpp
{

class Paraclean
{

public:

	double fis_;
	double threshold_;
	double error_ ;
	double prec_ ;
	double polypara_ ;
	bool dirichlet_ ;
	string optimizer_;

public:

	Paraclean();
	virtual ~Paraclean();

	double assignation(valarray<unsigned int> & geno, Stat & mystat);

	void set_fis(double f){fis_=f;}
	void set_threshold(double t){threshold_=t;}
	void set_dirichlet(bool d){dirichlet_=d;}
	void set_error(double e){error_=e;}
	void set_prec(double p){prec_=p;}
	void set_optimizer(string opt){optimizer_ = opt;}

private:
	double optimization0(Optimizer & optim, ParameterList & pl ,double xi);
	double optimization0dir(Optimizer & optim, ParameterList & pl , double xi, double alpha);
	double optimization1(Optimizer & optim, ParameterList & pl, double p1, double p2, double xi);
	double optimization1dir(Optimizer & optim, ParameterList & pl, double p1, double p2, double xi, double alpha);



};

} /* namespace bpp */
#endif /* PARACLEAN_H_ */
