/*
 * pfasclean.hpp
 *
 *  Created on: 15 avr. 2013
 *      Author: vcahais
 */

#ifndef PFASCLEAN_HPP_
#define PFASCLEAN_HPP_

#include <Bpp/Seq/Sequence.h>
#include <Bpp/Seq/Alphabet/DNA.h>

#include <vector>
#include <set>

using namespace std;
using namespace bpp;

namespace bpp
{

class Pfasclean
{

public:
	bool use_pfasclean_;
	unsigned int winsize_;
	double SNPdens_thrs_;
	double H_thrs_;

private:
	set<string> grp_species;
	vector<string> seq_species;
	vector <int> isfocal;

public:
	Pfasclean() : use_pfasclean_(false), winsize_(4), SNPdens_thrs_(0.4), H_thrs_(0.8){}
	virtual ~Pfasclean(){}

	void clean(vector<Sequence *> & group)
	{
		//get species names
		for (unsigned int i=0; i<group.size(); i++)
		{
			StringTokenizer st0(group[i]->getName(), "|", true, false);
			grp_species.insert(st0.getToken(1));
			seq_species.push_back(st0.getToken(1));
		}
		//clean by species
		set <string>::iterator it;
		for (it = grp_species.begin(); it != grp_species.end(); it++)
		{
			setIsFocal(*it);
			double win_SNPdens, win_H = 0.0;
			for(unsigned int i=0; (i+(3*winsize_)) < group[0]->size(); i+=3)
			{
				win_SNPdens=SNP_density(group, i, 3*winsize_);
				win_H=H_nb(group, i, 3*winsize_)/(win_SNPdens*3*winsize_*group.size());
				if(win_SNPdens>=SNPdens_thrs_ && win_H>=H_thrs_)
				  for(unsigned int j=i;j<i+3*winsize_;j++)
					  for (unsigned int seq=0; seq<group.size(); seq++)
						  group[seq]->setElement(j,14);
			}
		}
	}

private:
	/* is_SNP
	 * renvoie true si la position est un snp
	 */
	bool is_SNP(vector<Sequence *> & group, size_t pos)
	{
	  int ref=group[0]->getValue(pos);
	  for(unsigned int i=0; i<group.size(); i++)
	    if(isfocal[i])
	    	if (group[i]->getValue(pos) != 14 && group[i]->getValue(pos) != -1)
	    		if (group[i]->getValue(pos) != ref) return true;
	  return false;
	}

	/* SNP_density
	 *
	 */
	double SNP_density(vector<Sequence *> & group, unsigned int pos, unsigned int winsize)
	{
	  unsigned int nbsnp=0;
	  for(unsigned int i=pos; i< pos+winsize; i++)
	    if(is_SNP(group, i)) nbsnp++;
	  return (double)nbsnp/(double)winsize;
	}

	/* H_nb_pos
	 *
	 */
	unsigned int H_nb_pos(vector<Sequence *> & group, unsigned int pos)
	{
	  unsigned int hpos=0;
	  for(unsigned int i=0;i<group.size();i+=2)
		if(isfocal[i])
			if(group[i]->getValue(pos) != 14 && group[i]->getValue(pos) != -1)
				if(group[i]->getValue(pos)!=group[i+1]->getValue(pos)) hpos++;
	  return hpos;
	}

	/* H_nb
	 *
	 */
	unsigned int H_nb(vector<Sequence *> & group, unsigned int pos, unsigned int winsize)
	{
		unsigned int nbH=0;
		for(unsigned int i=pos;i<pos+winsize;i++)
			nbH+=H_nb_pos(group, i);
		return nbH;
	}

	void setIsFocal(string focale)
	{
		isfocal.clear();
		for (unsigned int i=0; i< seq_species.size(); i++)
		{
			if (seq_species[i]==focale) isfocal.push_back(1);
			else isfocal.push_back(0);
		}
	}



};

}



#endif /* PFASCLEAN_HPP_ */
