//============================================================================
// Name        : LnL.h
// Author      : Vincent Cahais, Nicolas Galtier
// Created on  : 17 septembre 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef LNL_H_
#define LNL_H_

#define MAX_NB_CLASSES 3

#include <vector>

#include "../Stat/Stat.h"
#include "Contigdata.h"
#include "Model.hpp"
#include "../Alr.hpp"

#include <Bpp/Numeric/Random/RandomTools.h>

using namespace std;

namespace bpp
{

class LnL
{
public:

	Stat * stat;
	AlrContig * contig;
	Model * model;
	Contigdata data;

	double mat_er[MAX_NB_CLASSES][4][4];
	double prior_er_cat[MAX_NB_CLASSES];
	double prior_er_prob[MAX_NB_CLASSES];
	double prior_aeb_cat[MAX_NB_CLASSES];
	double prior_aeb_prob[MAX_NB_CLASSES];

	unsigned int min_reads_per_ind;
	bool CALCULATE_POSTERIOR;
	bool DO_BEST;
	double fis_;

	vector<int> LNL_IND;
	vector< vector <vector <vector< vector<double> > > > >ln_cond_posterior_genotypes;

	LnL(Stat * s, AlrContig * c, Model * m, unsigned int n, double f);
	virtual ~LnL();

	double lnL(double er_alpha, double er_beta, double er_tstv, double aeb_alpha, double aeb_beta);
	void setModel(Model * m){model = m;}

private:
/*lnL*/


	double calculate_lnL_contig();
	double calculate_lnL_pos(Posdata * pos, Alrline * line, double aeb, int aeb_class, bool use_all_freq_err);
	double calculate_lnL_pos_ind(valarray<unsigned int> & values, int a, int b, double r_mat_er[4][4], double aeb, double log_multi_term);

	void calculate_posterior_genotypes();
	void calculate_posterior_genotypes_pos(Posdata * pos, Alrline * line, double aeb, int aeb_class, bool use_all_freq_err);

	void build_error_matrix(double ertot, double tstv, double mat_er[4][4]);
	double cond_log2(int a, int b);


};

}
#endif /* LNL_H_ */
