//============================================================================
// Name        : LnL_fct.hpp
// Author      : Vincent Cahais, Nicolas Galtier
// Created on  : 17 septembre 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef LNL_FCT_H_
#define LNL_FCT_H_

#define MIN_ERROR 0.000001
#define MIN_AEB 0.0001

#include <valarray>
#include "LnL.h"

#include <Bpp/Numeric/Function/Functions.h>
#include <Bpp/Numeric/Parameter.h>
//#include <Bpp/Numeric/Constraints.h>
//#include <limits>

using namespace std;
using namespace bpp;

namespace bpp
{

class LnL_fct: public virtual Function, public AbstractParametrizable
{
	private:
		double fval_;
		LnL calcul;

	public:

		LnL_fct(LnL & c) : AbstractParametrizable(""), fval_(0), calcul(c)
		{
			//version 2.0.2 de bpp-core
			//addParameter_(new Parameter("er_alpha", 1., new IntervalConstraint(0.05, NumConstants::PINF, 0, 0, NumConstants::TINY)));
			//addParameter_(new Parameter("er_beta", 100., new IntervalConstraint(1, 100/MIN_ERROR, 0, 0, NumConstants::TINY)));
			//addParameter_(new Parameter("er_tstv", 4.,new IntervalConstraint(0.05, NumConstants::PINF, 0, 0, NumConstants::TINY)));
			//addParameter_(new Parameter("aeb_alpha", 1., new IntervalConstraint(0.05, NumConstants::PINF, 0, 0, NumConstants::TINY)));
			//addParameter_(new Parameter("aeb_beta", 9., new IntervalConstraint(1, 100/MIN_AEB, 0, 0, 0.001)));

			addParameter_(Parameter("er_alpha", 1., new ExcludingPositiveReal(0.05)));
			addParameter_(Parameter("er_beta", 100., new ExcludingInterval(1., 100/MIN_ERROR)));
			addParameter_(Parameter("er_tstv", 4., new ExcludingPositiveReal(0.05)));
			addParameter_(Parameter("aeb_alpha", 1., new ExcludingPositiveReal(0.05)));
			addParameter_(Parameter("aeb_beta", 9., new ExcludingInterval(1., 100/MIN_AEB)));

		}

		LnL_fct* clone() const { return new LnL_fct(*this); }

	public:

		void setParameters(const ParameterList& pl)
		throw (ParameterNotFoundException, ConstraintException, Exception) {
			matchParametersValues(pl);
		}
		double getValue() const throw (Exception) { return fval_; }

		void fireParameterChanged(const ParameterList& pl) {
			double er_a = getParameterValue("er_alpha");
			double er_b = getParameterValue("er_beta");
			double er_ts = getParameterValue("er_tstv");
			double aeb_a = getParameterValue("aeb_alpha");
			double aeb_b = getParameterValue("aeb_beta");

			if (er_b > 1/MIN_ERROR) er_b = 1/MIN_ERROR;
			if (er_a > 1/MIN_AEB) er_a = 1/MIN_AEB;

			fval_ =-calcul.lnL(er_a, er_b, er_ts, aeb_a, aeb_b);
		}
};

}
#endif /* LNL_FCT_H_ */
