//============================================================================
// Name        : Posdata.cpp
// Author      : Vincent Cahais, Nicolas Galtier
// Created on  : 17 septembre 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#include "Posdata.h"

namespace bpp
{

Posdata::Posdata(unsigned int nb_ind) : allele_freq_all()
{
	// TODO Auto-generated constructor stub
	for (unsigned int i=0; i<nb_ind; i++)
		indiv.push_back(new ind);
}

Posdata::~Posdata()
{
	// TODO Auto-generated destructor stub
	for (unsigned int i=0; i<indiv.size(); i++)
		delete indiv[i];
}


bool Posdata::is_error_candidate(valarray<unsigned int> & values, unsigned int alnum, Stat & stat)
{
	double cum, max = 0.;
	unsigned int nbind = values.size() / 4;
	for (unsigned int i=0; i < nbind; i++)
	{
		if (values[(i*4)+alnum] == 0) continue;
		cum = stat.cumul_bino_proba(values[(i*4)+alnum], values.sum(), 0.5);
		if (cum > max) max = cum;
	}
	if (max < CANDIDATE_ERR_THR) return true; else return false;
}

void Posdata::calculate_allele_frequencies(valarray<unsigned int> & values, string species, Stat & stat)
{
	unsigned int total_number_of_reads = values.sum();
	unsigned int size = values.size();
	bool toremove[size];
	unsigned int allele_count[4] = {0,0,0,0};
	for (unsigned int i=0; i<values.size(); i++)
		allele_count[i%4] += values[i];
	if (total_number_of_reads == 0)
	{
		for (unsigned int i=0; i < 4; i++)
		{
			allele_freq_err[species].push_back(0.25);
			allele_freq_all[species].push_back(0.25);
		}
		return;
	}
	for (unsigned int i=0; i < 4; i++)
		allele_freq_all[species].push_back((double)allele_count[i]/total_number_of_reads);
	/* assume potential errors (allele of frequency below MAX_ERR_FREQ, or corresponding to a single read) */
	double freqtoremove = 0.;
	for (unsigned int i=0; i < 4; i++)
	{
		double alfreq = allele_freq_all[species][i];
		if (alfreq > 0. && is_error_candidate(values, i, stat))
		{
			toremove[i] = true;
			freqtoremove += alfreq;
		}
		else
		{
			toremove[i] = false;
		}
	}
	if (freqtoremove == 0.)
	{
		all_equals_err[species] = true;
		for (unsigned int i=0; i < 4; i++)
			allele_freq_err[species].push_back(allele_freq_all[species][i]);
	}
	else
	{
		all_equals_err[species] = false;
		for (unsigned int i=0; i < 4; i++)
		{
			if (toremove[i]) allele_freq_err[species].push_back(0.0);
			else allele_freq_err[species].push_back(allele_freq_all[species][i]/(1. - freqtoremove));
		}
	}
}

bool Posdata::check_no_error_possible(valarray<unsigned int> & values)
{
	for (unsigned int i=0; i < numberofindiv(); i++)
	{
		unsigned int nonzero_nb = 0;
		for (unsigned int j=0; j < 4; j++)
			if (values[(i*4)+j] > 0) nonzero_nb++;
		if (nonzero_nb > 2) return false;
	}
	return true;
}



} /* namespace bpp */
