//============================================================================
// Name        : Posdata.h
// Author      : Vincent Cahais, Nicolas Galtier
// Created on  : 17 septembre 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef POSDATA_H_
#define POSDATA_H_

#include "../Alr.hpp"
#include "../Stat/Stat.h"
#include <valarray>
#include <map>

#define MAX_NB_CLASSES 3
#define CANDIDATE_ERR_THR 0.01

namespace bpp
{

class Posdata
{

public:

	struct genotype
	{
		int allele[2];
		double prob;
		genotype(){allele[0]=100; allele[1]=100;}
	};

	struct ind
	{
		double log_multinomial_term;
		double posterior_genotypes_p[4][4];
		genotype best_genotype;
	};

	map<string, vector<double> > allele_freq_all;
	map<string, vector<double> > allele_freq_err;
	map<string, bool> all_equals_err;
	bool allele_freq_err_used;


	/* cond_posterior_er_p[a] = posterior proba distribution for er at this pos, conditional on aeb = a-th class for this contig */
	double posterior_er; 		        /* err estimate for this pos */
	double posterior_er_p[MAX_NB_CLASSES]; 	    /* unconditional posterior proba distribution for er at this pos */
	double cond_posterior_er_p[MAX_NB_CLASSES][MAX_NB_CLASSES];

	vector<ind*> indiv;

	Posdata(unsigned int nb_ind);
	virtual ~Posdata();

	unsigned int numberofindiv(){return indiv.size();}

	void calculate_allele_frequencies(valarray<unsigned int> & values,string species, Stat & stat);
	bool is_error_candidate(valarray<unsigned int> & values, unsigned int alnum, Stat & stat);
	bool check_no_error_possible(valarray<unsigned int> & values);


};

} /* namespace bpp */
#endif /* POSDATA_H_ */
