//============================================================================
// Name        : dmultinom.cpp
// Author      : Vincent Cahais, Sylvain Glemin
// Created on  : 13 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#include "Stat.h"

namespace bpp
{

double Stat::dmultinom(valarray<unsigned int> & values, valarray<double> & proba)
{
	if (values.size() != proba.size()) cerr << "dmultinom error : value.size != proba.size" << endl;

	unsigned int sum = values.sum();
	//normalize proba
	valarray<double> nproba = proba / proba.sum();
	//dmultinom = n! / (n1!*n2!*...*nx!) * p1^n1*p2^n2*...*px^nx

	double num = logfact(sum);		//numerator		(n!)
	double den = 0;					//denominator	((log(n1)!+log(n2)!+...+log(nx)!
	double pro = 0;    				//product		(n1*log(p1)+n2*log(p2)+...+nxlog(px))

	for (unsigned int i=0; i<values.size(); i++)
	{
		den += (logfact(values[i]));
		pro += values[i]* log(nproba[i]);
	}

	return exp(num-den+pro);
}

}
