//============================================================================
// Name        : Factorial.cpp
// Author      : Vincent Cahais
// Created on  : 13 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#include "Stat.h"

namespace bpp
{


/*******************************************************************************
 * LnGamma
 * returns ln(gamma(x)) for x > 0, accurate to 10 decimal places.
 * Stirling's formula is used for the central polynomial part of the procedure.
 * Pike MC & Hill ID (1966) Algorithm 291: Logarithm of the gamma function.
 * Communications of the Association for Computing Machinery, 9:684
 *******************************************************************************/

valarray<double> Stat::LnGamma (valarray<double> val)
{
	valarray<double> lng(val.size());
	for (unsigned int i=0; i<val.size(); i++)
	{
		lng[i] = LnGamma(val[i]);
	}
	return lng;
}


double Stat::LnGamma (unsigned int x)
{
	return logfact(x - 1);
}


double Stat::LnGamma (double x)
{
	double f = 0, fneg = 0, z, lng;
	int nx = (int)x;

	if (x==0) return 0;

	if ((double)nx == x && nx >= 0 && nx <= 11)
	{
		lng = logfact(nx - 1);
	}
	else
	{
		if (x <= 0)
		{
			cout << "LnGamma(" << x <<") not implemented" << endl;

			if ((int)x - x == 0) {
				cout << "lnGamma undefined\n";
				return(-1);
			}

			for (fneg = 1; x < 0; x++) { fneg /= x; }

			if (fneg < 0) {
				cout << "strange!! check lngamma" << endl;
				exit(0);
			}

			fneg = log(fneg);
		}

		if (x < 7)
		{
			f = 1;
			z = x - 1;
			while (++z < 7) f *= z;

			x = z;
			f =-log(f);
		}
		z = 1/(x*x);
		lng = fneg + f + (x-0.5)*log(x) - x + .918938533204673 + (((-.000595238095238*z+.000793650793651)*z-.002777777777778)*z +.083333333333333)/x;
	}

	return  lng;
}

/*******************************************************************************/
/*end*/
}
