//============================================================================
// Name        : Alr.h
// Author      : Vincent Cahais
// Created on  : 5 juillet 2012
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#ifndef UTILS_H_
#define UTILS_H_

#include <vector>
#include <map>
#include <iostream>
#include <Bpp/Text/TextTools.h>
#include <Bpp/Text/StringTokenizer.h>

using namespace std;

namespace bpp
{

class utils
{

public:

	utils(){};
	virtual ~utils(){};

	template <class T>
	static void printVector(valarray<T> & vec)
	{
		for (unsigned int i=0; i< vec.size(); i++)
			cerr << vec[i] << "\t";
		cerr << endl;
	}

	template <class T>
	static void printVector(vector<T> & vec)
	{
		for (unsigned int i=0; i< vec.size(); i++)
			cerr << vec[i] << "\t";
		cerr << endl;
	}

	template <class T>
	static void printMatrix(valarray <T> & vec)
	{
		for (unsigned int i=0; i< vec.size(); i++)
		{
			cerr << vec[i] << "\t";
			if (i%4 == 3) cerr << endl;
		}
		cerr << endl;
	}

	template <class T>
	static void printMatrix(vector <T> & vec)
	{
		for (unsigned int i=0; i< vec.size(); i++)
		{
			cerr << vec[i] << "\t";
			if (i%4 == 3) cerr << endl;
		}
		cerr << endl;
	}

	//sort matrix by decreasing sum of column value.
	static void sortMatrixByColumnSum(valarray<unsigned int> & matrix)
	{
		unsigned int nbindiv = matrix.size()/4;
		valarray<unsigned int> sum(4);
		for (unsigned int i = 0; i<4; i++)
		{
			valarray<unsigned int> foo = matrix[slice(i,nbindiv,4)];
			sum[i]=foo.sum();
		}

		for (unsigned int i = 1; i<4; i++)
		{
			valarray<unsigned int> foo = matrix[slice(i,nbindiv,4)];

		    unsigned int j = i;
		    while (j > 0 && sum[j-1] < sum[j])
		    {
		    	for (unsigned int n=0; n<nbindiv; n++)
		    		matrix[j+(n*4)] = matrix[j+(n*4)-1];
		    	unsigned int x = sum[j];
		    	sum[j] = sum[j-1];
		    	sum[j-1] = x;
		    	j = j-1;
		    }
		    for (unsigned int n=0; n<nbindiv; n++)
		    	matrix[j+(n*4)] = foo[n];
		}
	}

	static void getAllErrorRates(istream & err_file, map< string, vector<string> > & all_error_data)
	{
		string linebuffer;
		string contigname;
		while (!err_file.eof())
		{
			getline(err_file, linebuffer);
			if (linebuffer[0] == '#')
			{
				int pos = linebuffer.find('(')+1;
				int siz = linebuffer.find(')') - pos;
				contigname = linebuffer.substr(pos,siz);
			}
			else
			{
				//getErrorDataFromLine
				StringTokenizer st0 (linebuffer, ";", true, false);
				while(st0.hasMoreToken())
				{
					string data = st0.nextToken();
					int pos = data.find('=')+1;
					all_error_data[contigname].push_back(data.substr(pos,data.size()));
				}
			}
		}
	}

	/* tont */
	static char tont(int i)
	{
		if (i == 0) return 'A';
		else if (i == 1) return 'C';
		else if (i == 2) return 'G';
		else if (i == 3) return 'T';
		else return 'N';
	}

	static int toint(string n)
	{
		if (n == "A") return 0;
		else if (n== "C") return 1;
		else if (n == "G") return 2;
		else if (n == "T") return 3;
		else return 4;
	}

	static int toint(char n)
	{
		if (n == 'A') return 0;
		else if (n== 'C') return 1;
		else if (n == 'G') return 2;
		else if (n == 'T') return 3;
		else return 14;
	}

};

}

#endif /* UTILS_H_ */
