//============================================================================
// Name        : reads2snp.cpp
// Author      : Vincent Cahais, Nicolas Galtier, Sylvain Glemin, Aurélien Bernard, Benoit Nabholz, Etienne Loire
// Copyright   : http://www.cecill.info/licences/Licence_CeCILL-C_V1-fr.html
//============================================================================

#include <iostream>
#include <fstream>
#include "Stat/Stat.h"
#include "Alr.hpp"
#include "Reads2snp/Reads2snp.h"
#include "Paraclean/Logvrais0.hpp"
#include <Bpp/Phyl/Likelihood/PseudoNewtonOptimizer.h>
#include "Geno2pfas.hpp"
#include "Pfasclean.hpp"

using namespace std;
using namespace bpp;

string version = "1.2.1";
string version_date = "27-09-2012";
time_t rawtime = time(NULL);
time_t ttotdeb, ttotfin;
double tottime;

/**************************************************************************************/
/*debug function*/
/**************************************************************************************/
void debug()
{
	/*
	Stat mystat;

	mystat.fis = 0.99;
	mystat.set_error(0.00246587);
	mystat.prec_ = 0.001;

	unsigned int init1[] = {854,264,1,1,698,235,1,0,1520,450,3,1,1036,257,0,0,2378,488,2,3,2219,533,3,1,2852,533,1,1,730,225,0,0,778,208,2,1,884,233,2,1};
	unsigned int init2[] = {2,0,0,0,6,2,0,0,5,0,0,0,5,1,0,0,6,6,0,0,9,3,0,0,6,0,0,0,4,2,0,0,1,3,0,0,2,2,0,0};
	unsigned int init3[] = {4,3,0,0,8,6,0,1,6,6,0,0,6,5,0,0,7,10,0,0,13,8,0,0,6,5,0,0,6,5,1,0,1,7,0,0,3,4,0,0};

	unsigned int init4[] = {54,44,1,1,98,75,1,0};
	valarray<unsigned int> values(init2,40);

	unsigned int init6[] = {2,0,0,0};
	valarray<unsigned int> values2(init6,4);
	double init7[] = {0.99260239, 0.00246587, 0.00246587, 0.00246587};
	valarray<double> proba(init7,4);

	mystat.set_values(values);
	double xi = 0.7076923;

	cerr << endl;
	Logvrais0_dir log0(&mystat,0.001);
	ReparametrizationFunctionWrapper rpf0(&log0,false);
	ParameterList pl0 = rpf0.getParameters();
	ThreePointsNumericalDerivative tpnd0(&rpf0);
	tpnd0.setParametersToDerivate(pl0.getParameterNames());
	BfgsMultiDimensions optim0(&tpnd0);
	optim0.setVerbose(0);
	optim0.setProfiler(0);
	optim0.setMessageHandler(0);
	optim0.setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
	pl0.setParameterValue("xi",xi);
	optim0.init(pl0);
	double ml0 = optim0.optimize();
	cerr << "ml0=" << ml0 << " in R : 43.97648" << endl;
	optim0.getFunctionValue();
	cerr << "xi=" << optim0.getFunction()->getParameterValue("xi")<< endl;
	cerr << "logvrais0dir=" << mystat.logvrais0_dir(0.7076923, 0.99) << endl;

	cerr << endl;
	Logvrais1_dir log1(&mystat,0.001);
	ReparametrizationFunctionWrapper rpf1(&log1,false);
	ParameterList pl1 = rpf1.getParameters();
	ThreePointsNumericalDerivative tpnd1(&rpf1);
	tpnd1.setParametersToDerivate(pl1.getParameterNames());
	//BfgsMultiDimensions optim1(&tpnd1);
	PseudoNewtonOptimizer optim1(&tpnd1);
	optim1.setVerbose(0);
	optim1.setProfiler(0);
	optim1.setMessageHandler(0);
	optim1.setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
	pl1.setParameterValue("xi",xi);
	pl1.setParameterValue("p1",0.999);
	pl1.setParameterValue("p2",0.001);
	optim1.init(pl1);
	double ml1 = optim1.optimize();
	cerr << "ml1=" << ml1 << "\tin R : 105.8411" << endl;
	cerr << "p1=" << optim1.getFunction()->getParameterValue("p1")<< "\tin R : 0.5999006" << endl;
	cerr << "p2=" << optim1.getFunction()->getParameterValue("p2")<< "\tin R : 0.6335156" << endl;
	cerr << "xi=" << optim1.getFunction()->getParameterValue("xi")<< "\tin R : 0.8031929" << endl;
	//calcul of likelihood whith R returned parameters values
	cerr << "logvrais1=" << mystat.logvrais1_dir(0.999, 0.1442293, 0.6736401, 99.9999166) << " in R : 105.8411" << endl;
	*/
	/*cerr << endl;
	Logvrais1_dir log1dir(&mystat,0.001);
	//ParameterList pl1dir = log1dir.getParameters();
	ReparametrizationFunctionWrapper rpf1dir(&log1dir,false);
	ParameterList pl1dir = rpf1dir.getParameters();
	ThreePointsNumericalDerivative tpnd1dir(&rpf1dir);
	tpnd1dir.setParametersToDerivate(pl1dir.getParameterNames());
	PseudoNewtonOptimizer optim1dir(&tpnd1dir);
	optim1dir.setVerbose(0);
	optim1dir.setProfiler(0);
	optim1dir.setMessageHandler(0);
	optim1dir.setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
	pl1dir.setParameterValue("xi",0.8028201);
	pl1dir.setParameterValue("p1",0.001);
	pl1dir.setParameterValue("p2",0.999);
	pl1dir.setParameterValue("alpha",100);
	optim1dir.init(pl1);
	double ml1dir = optim1dir.optimize();
	cerr << "ml1dir=" << ml1dir << "\tin R : 98.33069" << endl;
	cerr << "p1=" << optim1dir.getFunction()->getParameterValue("p1")<< "\tin R : 0.5991741" << endl;
	cerr << "p1=" << log1.getParameterValue("p1")<< endl;
	cerr << "p2=" << optim1dir.getFunction()->getParameterValue("p2")<< "\tin R : 0.6267094" << endl;
	cerr << "p2=" << log1.getParameterValue("p2")<< endl;
	cerr << "xi=" << optim1dir.getFunction()->getParameterValue("xi")<< "\tin R : 0.7987207" << endl;
	cerr << "xi=" << log1.getParameterValue("xi")<< endl;
	cerr << "alpha=" << optim1dir.getFunction()->getParameterValue("alpha")<< "\tin R : 240.3285593" << endl;
	//calcul of likelihood whith R returned parameters values
	cerr << "logvrais1dir=" << mystat.logvrais1_dir(0.5991741, 0.6267094, 0.7987207, 240.3285593) << " in R : 98.33069" << endl;

	ReparametrizationFunctionWrapper rpf2(&log1,false);
	ParameterList pl2 = rpf2.getParameters();
	//ParameterList pl2 = log1.getParameters();
	DownhillSimplexMethod optim2(&rpf2);
	optim2.setVerbose(0);
	optim2.setProfiler(0);
	optim2.setMessageHandler(0);
	optim2.setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
	pl2.setParameterValue("xi",0.8028201);
	pl2.setParameterValue("p1",0.001);
	pl2.setParameterValue("p2",0.999);
	optim2.init(pl2);
	double ml2 = optim2.optimize();
	cerr << "ml1dir=" << ml2 << endl;
	cerr << "p1=" << optim2.getFunction()->getParameterValue("p1")<< endl;
	cerr << "p2=" << optim2.getFunction()->getParameterValue("p2")<< endl;
	cerr << "xi=" << optim2.getFunction()->getParameterValue("xi")<< endl;
	cerr << "logvrais2=" << mystat.logvrais1(0.2335, 0.4995, 0.5) << endl;*/
	exit(0);
}

/**************************************************************************************/
/*version*/
/**************************************************************************************/
void display()
{
   cout << "reads2snp version = " << version << "  " << version_date << endl;
   exit(0);
}

/**************************************************************************************/
/*usage*/
/**************************************************************************************/
void usage()
{
	cout << endl;
	cout << "usage: reads_to_snps [options] -alr alr_file" << endl;
	cout << endl;
	cout << "alr_file : aligned sequence file in ALR format" << endl;
	cout << endl;
	cout << "options:" << endl;

	cout << "[-gen string]            output file name(alr_gen)                        default=alr_file.alr_gen" << endl;
	cout << "[-dat string]            output data file name(dat)                       default=alr_file.dat" << endl;
	cout << "[-sh|-sep|-all]          parameters_sharing: shared|separated|all         default=all      "<< endl;
	cout << "[-best|-rand]            output_genotype:    best|random                  default=best     "<< endl;
	cout << "[-acc|-tol|-for]         multi_alleles:      accepted|tolerated|forbidden default=tolerated"<< endl;
	cout << "[-min positive integer]  min_coverage                                     default=-1		"<< endl;
	cout << "[-fis positive float]    fis                                              default=0.0      "<< endl;
	cout << "[-M1|-M2]                models: M1= error rate                           default=M1       "<< endl;
	cout << "                                 M2= error rate + allelic expression bias                  "<< endl;
	cout << endl;
	cout << "paraclean" << endl;
	cout << "[-par]                   use paraclean       remove paralogous positions                   "<< endl;
	cout << "[-dir]                   use dirichlet                                                     "<< endl;
	cout << "[-thr positive float]    likelihood threshold  ]0,1[                      default=0.001    "<< endl;
	cout << "[-spa]                   skip Paralogous :   skip contigs with paralogous positions.       "<< endl;
	cout << "[-pre]                   precision on parameters                          default=0.001    "<< endl;
	cout <<	"[-kun]                   keep position when paraclean return an error                      "<< endl;
	cout << "[-opt]                   optimizer used for paraclean [bfgs|newton]       default=newton   "<< endl;
	cout << "[-err]                   transition/tranvertion error ]0,0.333[. If not set the value is calculated for each contigs" << endl;
	cout << endl;
	cout << "geno2pfas" << endl;
	cout << "[-gpv positive float]    geno2pfas pvalue threshold ]0,1[                 default=0.95     "<< endl;
	cout << endl;
	cout << "pfasclean" << endl;
	cout << "[-pfc]                   use pfasclean       hide non reliable zones                       "<<endl;
	cout << "[-win]                   windows size                                     default=4        "<<endl;
	cout << "[-het]                   heterozygotie density threshold                  default=0.8      "<<endl;
	cout << "[-snp                    snp density threshold                            default=0.4      "<<endl;
	cout << endl;

	exit(0);
}

/**************************************************************************************/
/*printtime function*/
/**************************************************************************************/
void printtime(double rtime)
{

	int d, h, m, s;

	d = (int)rtime/86400;
	rtime -= d*86400;
	if (d) printf("%d day", d);
	if (d > 1) printf("s");
	if (d) printf(" ");

	h = (int)rtime/3600;
	rtime -= h*3600;
	if (h) printf("%d hour", h);
	if (h > 1) printf("s");
	if (h) printf(" ");

	m = (int)rtime/60;
	rtime -= m*60;
	if (m) printf("%d minute", m);
	if (m > 1) printf("s");
	if (m) printf(" ");

	s = (int)rtime;
	if (s) printf("%d second", s);
	if (s > 1) printf("s");

	if (d == 0 && h == 0 && m == 0 && s == 0) printf("less than 1 second");

	printf("\n");
}


/**************************************************************************************/
/*MAIN function*/
/**************************************************************************************/
int main(int argc, char** argv)
{
	//arguments
	string argument;
	string alrfile    		 = ""; // INPUT  : alr file name (.alr)
	string alrgenfile 		 = ""; // OUTPUT : alr file name (alr.gen)
	string species_data_file = ""; // OUTPUT : data file name (.data)
	string alleles_file      = ""; // OUTPUT : alleles file (.fasta)
	//string outfname_aeb    = ""; // OUTPUT : aeb file (.aeb)
	//string outfname_err    = ""; // OUTPUT : err file (.err)

	//main objects
	Stat mystat;
	Reads2snp r2s(&mystat);
	Pfasclean pfc;
	Geno2pfas g2p;

	//parse options
	for (int i=1; i<argc ; i++)
	{
		argument = argv[i];
		if (argument == "-help" || argument == "-h") 	usage();
		if (argument == "-v") 	display();
		if (argument == "-debug") debug();

		if (argument == "-alr") 	alrfile    = argv[i+1];
		if (argument == "-gen") 	alrgenfile = argv[i+1];
		if (argument == "-dat") 	species_data_file = argv[i+1];
		if (argument == "-all")     alleles_file = argv[i+1];
		//if (argument == "-aeb") 	outfname_aeb = argv[i+1];
		//if (argument == "-err") 	outfname_err = argv[i+1];

		/*read2snp*/
		//if (argument == "-M0"  ) {r2s.domodel(0); r2s.chosen_model = 0;}
		if (argument == "-M1" ) {r2s.domodel(1); r2s.chosen_model = 1;}
		if (argument == "-M2" ) {r2s.domodel(2); r2s.chosen_model = 2;}

		if (argument == "-sh"  || argument == "-all")  {r2s.do_sh  = true;}
		if (argument == "-sep" || argument == "-all")  {r2s.do_sep = true;}

		if (argument == "-best") r2s.do_best = true;
		if (argument == "-rand") r2s.do_best = false;

		if (argument == "-acc") r2s.MULTI_ALLELES = "accepted";
		if (argument == "-tol") r2s.MULTI_ALLELES = "tolerated";
		if (argument == "-for") r2s.MULTI_ALLELES = "forbidden";

		if (argument == "-min") r2s.min_reads_per_ind = atoi(argv[i+1]);

		/*paraclean*/
		if (argument == "-fis") r2s.fis_       = atof(argv[i+1]);
		if (argument == "-thr") r2s.threshold_ = atof(argv[i+1]);
		if (argument == "-dir") r2s.dirichlet_ = true;
		if (argument == "-spa") r2s.skipParalogous_ = true;
		if (argument == "-pre") r2s.prec_ = atof(argv[i+1]);
		if (argument == "-kun") r2s.keepunresolved_ = true;
		if (argument == "-opt") r2s.optimizer_ = argv[i+1];
		if (argument == "-err") r2s.error_ = atof(argv[i+1]);

		if (argument == "-par") r2s.use_paraclean = true;

		//geno2pfas
		if (argument == "-gpv") g2p.threshold_ = atof(argv[i+1]);

		//pfasclean
		if (argument == "-pfc") pfc.use_pfasclean_ =true;
		if (argument == "-win") pfc.winsize_ = atoi(argv[i+1]);
		if (argument == "-het") pfc.H_thrs_ = atof(argv[i+1]);
		if (argument == "-snp") pfc.SNPdens_thrs_ = atof(argv[i+1]);

	}

	//test parameters

	//if no model is selected, all are used by default
	if (!r2s.getnumberofactivemodels()) {
		r2s.domodel(1);
		r2s.chosen_model = 1;
	}

	//files names
	if (alrfile == "") usage();

	// values
	if (r2s.fis_ < 0 || r2s.fis_ > 1) usage();
	if (r2s.threshold_ < 0 || r2s.threshold_ > 1) usage();
	if (r2s.min_reads_per_ind < 0) usage();
	if (r2s.error_ < 0 || r2s.error_ > 0.333) usage();

	//optimizer
	if (r2s.optimizer_ != "bfgs" || r2s.optimizer_ != "newton")
	{
		cout << "warning : " << r2s.optimizer_ << " optimizer doesn't exist, default is used" << endl;
		r2s.optimizer_ = "newton";
	}


	/**************************************************************************************/
	// Welcome message
	cout << "################################################################" << endl;
	cout << "# Welcome in Reads2snp (Version " << version << " (Last update: " << version_date << ")) #" << endl;
	cout << "################################################################" << endl;
	cout << "Start time: " << ctime(&rawtime) << endl;

	ttotdeb = time(NULL);

	//read alr file
	ifstream alr(alrfile.c_str(), ios::in);
	Alrparser alreader;

	//open files
	string basename = alrfile.erase(alrfile.rfind('.'));
	if (alrgenfile == "") alrgenfile = basename + ".gen";
	std::ofstream alrgen(alrgenfile.c_str(), std::ios::out | std::ios::trunc);
	if (species_data_file == "") species_data_file = basename +".data";
	ofstream data (species_data_file.c_str(), ios::out | ios::trunc);
	if (alleles_file == "") alleles_file = basename +".fasta";
	ofstream alleles (alleles_file.c_str(), ios::out | ios::trunc);
	//if (outfname_aeb == "") outfname_aeb = alrgenfile + ".aeb";
	//std::ofstream outfile_aeb (outfname_aeb.c_str(), ios::out | ios::trunc);
	//if (outfname_err == "") outfname_err = alrgenfile + ".err";
	//std::ofstream outfile_err (outfname_err.c_str(), ios::out | ios::trunc);

	//read2snp is run for each contig in alr file
	AlrContig * contig = new AlrContig;
	while(alreader.nextsequence(alr, *contig))
	{
		//read next contig in alr file
		cout << ">" << contig->name << endl;
		cout << "Found ";
		cout << contig->numberofspecies() << " species, ";
		cout << contig->numberofindiv()   << " individuals, ";
		cout << contig->size()            << " positions",
		cout << endl;

		//if number of species = 1, shared option is nonsense.
		if (contig->numberofspecies() == 1) r2s.do_sep = false;
		//if do_sep is false, do_sh must be true.
		if (r2s.do_sep == false) r2s.do_sh = true;

		//compute
		r2s.compute(*contig);
		//Write alrgen file (.alr.gen)
		string sctg = r2s.write_genotypes(*contig, alrgen);
		//Write all error_rates/bias data into a specific output file (.data)
		r2s.write_speciesdata(data);
		//write alleles sequences
		if (sctg != "")
		{
			vector<Sequence*> sequences;
			g2p.gen2seq(sctg, sequences);
			if (pfc.use_pfasclean_) pfc.clean(sequences);
			g2p.write_alleles(sequences, alleles);
		}
		//Write logs
		int heterozygous = r2s.outlog.nb_positions - ( r2s.outlog.nb_homozygous + r2s.outlog.nb_hidden_positions );
		cout << "read2snp: ";
		cout << r2s.outlog.nb_positions << " positions, ";
		cout << r2s.outlog.nb_homozygous << " homozygous (";
		cout << (double)r2s.outlog.nb_homozygous/(double)r2s.outlog.nb_positions*100 << "%), ";
		cout << heterozygous << " hétérozygous (";
		cout << (double)heterozygous/(double)r2s.outlog.nb_positions*100 << "%), ";
		cout << r2s.outlog.nb_hidden_positions << " Paralogous positions (";
		cout << (double)r2s.outlog.nb_hidden_positions/(double)r2s.outlog.nb_positions*100 << "%) " << endl;
		cout << "paraclean: ";
		cout << r2s.outlog.nb_checked_positions << " Checked positions, ";
		cout << r2s.outlog.nb_position_in_error << " Position in error, ";
		cout << endl << endl;



		//Write output file for allelic expression bias data
		//cout << "Write allelic expression bias data" << endl;
		//r2s.write_aeb(*contig, outfile_aeb);
		//cout << "    --> Contig-specific estimated allelic expression bias written in file: " <<  outfname_aeb << endl;

		//Write output file for Estimated average error rates data
		//cout << "Write Estimated average error rates data" << endl;
		//r2s.write_err(*contig, outfile_err);
		//cout << "    --> Position-specific estimated error rates written in file: " << outfname_err << endl;

		delete contig;
		contig = new AlrContig;
	}
	delete contig;

	//close files
	alr.close();
	alrgen.close();
	data.close();
	//outfile_aeb.close();
	//outfile_err.close();

	ttotfin = time(NULL);
	tottime = difftime(ttotfin, ttotdeb);
	cout << "Running time: ";
	printtime(tottime);
	cout << endl;

	// Goodbye message
	time_t end_rawtime = time(NULL);
	cout << "Stop time: " << ctime(&end_rawtime);
	cout << "################################################################" << endl;
	cout << "#                 Reads2snp - End of execution                 #" << endl;
	cout << "################################################################" << endl;

	return 0;
}
