/*
 * OSite.h
 *
 *  Created on: 11 mai 2012
 *      Author: vcahais
 */

#ifndef OSITE_H_
#define OSITE_H_

#include <vector>
#include <map>
#include <set>

using namespace std;

class OSite
{

private:

	OSite* prev_;		// previous node
	OSite* next_;		// next node
	char consensus_;	// major nucleotide at this position

	vector <unsigned int> sequences_;		// index sequences;
	map <unsigned int, char> nucleotides_;	// get nucleotide by sequence index;
	unsigned int number_[6];				// count nucleotides;

public:

	OSite(){ prev_ = 0; next_ = 0;for (unsigned int i=0; i<6; i++) number_[i]=0; consensus_='E';}
	virtual ~OSite(){delete next_;}

	/*******************************************************************************/
	/*nodes*/
	/*******************************************************************************/
	void setNext(OSite * node){next_=node;}
	OSite * getNext(){ return next_;}
	void setPrev(OSite * node){prev_=node;}
	OSite * getPrev(){ return prev_;}

	/*******************************************************************************/
	/*isNodeFor : only for DEBUG !slow!*/
	/*******************************************************************************/
	bool isNodeFor(unsigned int index)
	{
		for (unsigned int i=0; i< sequences_.size(); i++)
		{
			if (sequences_[i]==index) return true;
			if (sequences_[i] > index) return false;
		}
		return false;
	}

	/*******************************************************************************/
	/*nucleotides :*/
	/*******************************************************************************/
	unsigned int getNumberOf(char c){return number_[char2int(c)];}
	char getNucleotide(unsigned int index) {return nucleotides_.at(index);}

	//!!!ALWAYS RUN getVariability BEFORE getConsensus!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	char getConsensus()	{return consensus_;	}
	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


	/*******************************************************************************/
	/*sequences :*/
	/*******************************************************************************/
	unsigned int getNumberOfSequence(){return nucleotides_.size();}
	unsigned int getSequence(unsigned int i){return sequences_[i];}
	void addSequence(unsigned int index, char c)
	{
		//if (nucleotides_.find(index) == nucleotides_.end())
		{
			sequences_.push_back(index);
			nucleotides_[index]=c;
			number_[char2int(c)]++;
		}
	}

	/*******************************************************************************/
	/*getVariability : return number of different nucleotide*/
	/*******************************************************************************/
	unsigned int getVariability()
	{
		unsigned int variability=0;
		unsigned int c = char2int(getNucleotide(0));
		unsigned int max = number_[c];
		for (unsigned int i=0; i<6; i++)
		{
			if (number_[i] > 0 and i < 5) variability++;
			if (number_[i] > max)
			{
				c = i;
				max = number_[i];
			}
		}
		consensus_= int2char(c);
		return variability;
	}

private:

	/*******************************************************************************/
	/* char2int and int2char tables*/
	/*******************************************************************************/
	unsigned int char2int(char c)
	{
		switch(c)
		{
			case 'A': return 0; break;
			case 'C': return 1; break;
			case 'G': return 2; break;
			case 'T': return 3; break;
			case '-': return 4; break;
			default : return 5;
		}
	}

	unsigned int int2char(unsigned int c)
	{
		switch(c)
		{
			case 0: return 'A'; break;
			case 1: return 'C'; break;
			case 2: return 'G'; break;
			case 3: return 'T'; break;
			case 4: return '-'; break;
			default : return 'N';
		}
	}


};

#endif /* OSITE_H_ */
