/*
 * Sam2alr.cpp
 *
 *  Created on: 15 nov. 2012
 *      Author: cahais
 */

#include "Sam2alr.h"

using namespace std;
using namespace bpp;

Sam2alr::Sam2alr()
{
	// TODO Auto-generated constructor stub
}

Sam2alr::~Sam2alr()
{
	// TODO Auto-generated destructor stub
}

/*******************************************************************************/
/*clean : delete trees after each contig */
/*******************************************************************************/
void Sam2alr::Sam2alr::clean()
{
	delete tree_[0];
	tree_.clear();
	map<string, OSite* >::iterator it;
	for (it=indiv_trees_.begin(); it != indiv_trees_.end(); it++)
		delete it->second;
	indiv_trees_.clear();
}

/*******************************************************************************/
/*getNodeByOrder : return nodes position */
/*******************************************************************************/
OSite* Sam2alr::getNodeByOrder(unsigned int pos){return tree_[pos];}
OSite* Sam2alr::getNodeByOrder(unsigned int pos, OSite * root)
{
	OSite * curr=root;
	for (unsigned int i=0; i<pos; i++)
		 curr = curr->getNext();
	return curr;
}

/*******************************************************************************/
/*initialize : create main tree */
/*******************************************************************************/
void Sam2alr::initialize(OSite * node, BasicSequence & seq)
{
	OSite * next=0;
	for (unsigned int i=0; i< seq.size(); i++)
	{
		next = new OSite;
		node->setNext(next);
		char c = seq.getChar(i)[0];
		next->addSequence(0,c);
		tree_.push_back(next);
		node = next;
	}
}

/*******************************************************************************/
/*newTree : create individual tree */
/*******************************************************************************/
OSite* Sam2alr::newTree(string indiv)
{
	indiv_trees_[indiv] = new OSite;
	OSite * curr = indiv_trees_[indiv];
	for(unsigned int i=0; i<tree_.size(); i++)
	{
		curr->setNext(new OSite);
		curr = curr->getNext();
	}
	return indiv_trees_[indiv];
}

/*******************************************************************************/
/*samline2OSiteStrain : fill tree with a samline */
/*******************************************************************************/
void Sam2alr::samline2OSiteStrain(Sam::Samline & saml, OSite * node, unsigned int index)
{
	string chiffre = "";
	size_t pos = 0;
	//cout << saml.cigar << " / " << saml.seq.size() << endl;

	for (size_t i=0; i< saml.cigar.size(); i++)
	{
		if ((int)saml.cigar[i] > 47 and (int)saml.cigar[i] < 58)
		{
			chiffre+=saml.cigar[i];
		}
		else
		{
			size_t nbpos = atoi(chiffre.c_str());
			//cout << "case:" << saml.cigar[i] << " / pos:" << pos << " / nbpos:" << nbpos << endl;
			switch (saml.cigar[i])
			{
				case 'M': // cas d'un match
					for (unsigned int i=0; i<nbpos; i++){
						node->addSequence(index,saml.seq[pos+i]);
						if (node->getNext()) node = node->getNext();
						else break;
					}
					pos+=nbpos;
					break;
				case 'D': // cas d'une deletion
					for (unsigned int i=0; i<nbpos; i++){
						node->addSequence(index,'-');
						if (node->getNext()) node = node->getNext();
						else break;
					}
						//pos+=nbpos;
					break;
				case 'I': // cas d'une insertion
					//if (insertion && node->getSon('-'))
						//TODO case insertion
					pos+=nbpos;
					break;
				case 'S': // ?
					/*for (unsigned int i=0; i<nbpos; i++){
						node->addSequence(index,saml.seq[pos+i]);
						if (node->getNext()) node = node->getNext();
						else break;
					}*/
					pos+=nbpos;
					break;
				default : cerr << "wrong cigar string : " << saml.cigar << endl; break;
			}
			chiffre = "";
		}
	}
}

/*******************************************************************************/
/*writeContig : write corrected contig sequence */
/*******************************************************************************/
void Sam2alr::writeContig(ostream & out)
{
	OSite * curr = tree_[0]->getNext();
	unsigned int cpt=0;
	while(curr->getNext())
	{
		if (curr->getConsensus() != '-')
		{
			cpt++;
			out << curr->getNucleotide(0);
			if ( cpt%60 == 0) out << endl;
		}
		curr = curr->getNext();
	}
	out << curr->getNucleotide(0) << endl;
}

/*******************************************************************************/
/*writeAlignment : DEBUG display read alignment !increase time execution!*/
/*******************************************************************************/
void Sam2alr::writeAlignment(OSite * root, unsigned int nbseq)
{
	for (unsigned int i=0; i< nbseq; i++)
	{
		OSite * curr = root->getNext();
		while (curr->getNext())
		{
			if (curr->isNodeFor(i))
				cout << curr->getNucleotide(i);
			else cout << "-";
			curr = curr->getNext();
		}
		cout << endl;
	}
}

/*******************************************************************************/
/*run : */
/*******************************************************************************/
void Sam2alr::run(ifstream & reffile, ifstream & samfile, ofstream & out,
		 ofstream & fasta, ofstream & covfile)
{
	//cout << "start " << endl;
	Sam samReader;
	samReader.build(samfile);
	Sam::Samline samline;
	//cout << "indexing samfile ok" << endl;

	CustomNucleicAlphabet alpha;
	Fasta fastaReader;
	vector<ios::pos_type>::iterator it;
	BasicSequence bs("","",&alpha);

	while(!reffile.eof())
	{
		vector<string> sequences;
		OSite * curr =0;
		//read sequence
		fastaReader.nextSequence(reffile,bs);
		if (bs.size() < minsize_)
		{
			cout << bs.getName() << " too small " << bs.size() << endl;
			continue;
		}
		sequences.push_back(bs.getName());
		//cout << "reference : " << bs.getName() << endl;
		//cout << "reference size : " << bs.size() << endl;
		//init main tree
		tree_.push_back(new OSite);
		initialize(tree_[0],bs);
		//cout << "tree : " << tree_.size() << endl;

		//lecture du fichier sam pour la sequence de reference en cours
		double cover = 0;
		unsigned int nbreads =0;
		vector<std::ios::pos_type> pos = samReader.getPositions(bs.getName());
		for (it=pos.begin(); it<pos.end(); it++)
		{
			//read samline
			samfile.seekg(*it);
			samReader.readSamLine(samfile, samline);
			//Pour le calcul de la couverture
			nbreads++;
			cover+=samline.seq.size();
			//TEST si cigar contient un S
			if (samline.cigar.find('S') != string::npos) continue;
			//TEST sur la position du read
			if (samline.pos + samline.seq.size() > bs.size() + (0.1*samline.seq.size())) continue;
			//resolve name and register sequence
			unsigned int index = sequences.size();
			StringTokenizer qname(samline.qname, "|", true, false);
			sequences.push_back(qname.getToken(2));
			//add sequence to the main tree
			curr = getNodeByOrder(samline.pos);
			samline2OSiteStrain(samline, curr, index );
			//add sequence to individual tree
			string name = qname.getToken(0) + "|" + qname.getToken(1);
			curr = indiv_trees_[name];
			if (!curr) curr = newTree(name);
			curr = getNodeByOrder(samline.pos, curr);
			samline2OSiteStrain(samline, curr, index);
		}

		//WRITE cover
		double cov = cover/(double)bs.size();
		covfile << bs.getName() << "\t" << bs.size() << "\t" << cov << "\t" << nbreads << endl;
		if (cov > mincov_)
		{
			//WRITE alignement in alr format;
			Alr alrWriter;
			alrWriter.toAlr(tree_[0],bs.getName(),indiv_trees_, out);

			//WRITE contig in fasta format
			fasta << ">" << bs.getName() << endl;
			writeContig(fasta);
			//writeAlignment(tree_[0],sequences.size());
		}
		else{cout << bs.getName() << " low cover : " << cov << endl;}

		//CLEAN trees
		clean();
	}
}

/*******************************************************************************/
/*run : */
/*******************************************************************************/
void Sam2alr::run(ifstream & reffile, ofstream & out, ofstream & fasta, ofstream & covfile)
{
	//cout << "start " << endl;
	Sam::Samline samline;
	CustomNucleicAlphabet alpha;
	Fasta fastaReader;
	vector<ios::pos_type>::iterator it;
	BasicSequence bs("","",&alpha);

	vector <Sam*> samReaders;
	for (unsigned int i=0; i< samfiles_.size(); i++)
	{
		saminputs_.push_back(new ifstream(samfiles_[i].c_str(), ios::in));
		samReaders.push_back(new Sam);
		samReaders[i]->build(*saminputs_[i]);
	}
	while(!reffile.eof())
	{
		vector<string> sequences;
		OSite * curr =0;
		//read sequence
		fastaReader.nextSequence(reffile,bs);
		if (bs.size() < minsize_)
		{
			cout << bs.getName() << " too small " << bs.size() << endl;
			continue;
		}
		sequences.push_back(bs.getName());
		//cout << "reference : " << bs.getName() << endl;
		//cout << "reference size : " << bs.size() << endl;
		//init main tree
		tree_.push_back(new OSite);
		initialize(tree_[0],bs);
		//cout << "tree : " << tree_.size() << endl;

		//lecture des fichier sam pour la sequence de reference en cours
		double cover = 0;
		unsigned int nbreads =0;
		for (unsigned int i=0; i< samfiles_.size(); i++)
		{
			//lecture du fichier sam pour la sequence de reference en cours
			vector<std::ios::pos_type> pos = samReaders[i]->getPositions(bs.getName());
			for (it=pos.begin(); it<pos.end(); it++)
			{
				//read samline
				saminputs_[i]->seekg(*it);
				samReaders[i]->readSamLine(*saminputs_[i], samline);
				//Pour le calcul de la couverture
				nbreads++;
				cover+=samline.seq.size();
				//TEST si cigar contient un S
				if (samline.cigar.find('S') != string::npos) continue;
				//TEST sur la position du read
				if (samline.pos + samline.seq.size() > bs.size() + (0.1*samline.seq.size())) continue;
				//resolve name and register sequence
				unsigned int index = sequences.size();
				StringTokenizer qname(samline.qname, "|", true, false);
				//Test si qname au format popphyl
				string name="";
				if (qname.numberOfRemainingTokens() > 2)
				{
					sequences.push_back(qname.getToken(2));
					name = qname.getToken(0) + "|" + qname.getToken(1);
				}
				else if (species_.size() > i)
				{
					sequences.push_back(samline.qname);
					name = species_[i] + "|" + indiv_[i];
				}
				else
				{
					cerr << "species name and/or individuals name is/are not provided" << endl;
					exit(0);
				}
				//add sequence to the main tree
				curr = getNodeByOrder(samline.pos);
				samline2OSiteStrain(samline, curr, index );
				//add sequence to individual tree
				curr = indiv_trees_[name];
				if (!curr) curr = newTree(name);
				curr = getNodeByOrder(samline.pos, curr);
				samline2OSiteStrain(samline, curr, index);
			}
		}

		//WRITE cover
		double cov = cover/(double)bs.size();
		covfile << bs.getName() << "\t" << bs.size() << "\t" << cov << "\t" << nbreads << endl;
		if (cov > mincov_)
		{
			//WRITE alignement in alr format;
			Alr alrWriter;
			if (indiv_trees_.size() > 0)
			{
				alrWriter.toAlr(tree_[0],bs.getName(),indiv_trees_, out);

				//WRITE contig in fasta format
				fasta << ">" << bs.getName() << endl;
				writeContig(fasta);
				//writeAlignment(tree_[0],sequences.size());
			}
		}
		else{cout << bs.getName() << " low cover : " << cov << endl;}

		//CLEAN trees
		clean();
	}
}


///return 0 if error, 1 if list of alr files, 2 if sam file;
int Sam2alr::readNames(ifstream & filenames)
{
	string firstline;
	string linebuffer;

	getline(filenames, firstline);
	StringTokenizer line(firstline, "\t ", false, false);
	if (line.numberOfRemainingTokens() > 11 ) return 2;
	if (firstline == "[samfiles]" )
	{
		while(!filenames.eof())
		{
			getline(filenames, linebuffer);
			StringTokenizer line(linebuffer, "\t ", false, false);
			if (line.numberOfRemainingTokens() == 3)
			{
				samfiles_.push_back(line.nextToken());
				species_.push_back(line.nextToken());
				indiv_.push_back(line.nextToken());
			}
			else if (line.numberOfRemainingTokens() == 0){}
			else return 0;
		}
	}
	return 1;
}


void Sam2alr::error_wrongSamInputFormat()
{
	cerr << "wrong input : files names" << endl << endl;
	cerr << "[samfiles]" << endl;
	cerr << "/path/myfile1.sam	species1	individual1" << endl;
	cerr << "/path/myfile2.sam	species2	individual2" << endl;
	cerr << "/path/myfile3.sam	species3	individual3" << endl;
	exit(0);
}
