/*
 * Sam2alr.hpp
 *
 *  Created on: 15 nov. 2012
 *      Author: cahais
 */

#ifndef SAM2ALR_H_
#define SAM2ALR_H_

#include <vector>
#include <iostream>
#include "OSite.h"

#include <Bpp/Text/StringTokenizer.h>
#include <Bpp/Seq/Io/Fasta.h>
#include <Bpp/Seq/SequenceWithQuality.h>

#include <ppc/Seq/Io/Sam.h>
#include <ppc/Seq/Io/Qual.h>
#include <ppc/Seq/Alphabet/CustomNucleicAlphabet.h>

#include "Alr.h"

using namespace std;
using namespace ppc;

namespace bpp
{

class Sam2alr
{

private:
//trees
vector<OSite * > tree_;
map<string, OSite* > indiv_trees_;

//filenames
vector<string> samfiles_;
vector<ifstream*> saminputs_;
vector<string> species_;
vector<string> indiv_;

//parameters
double mincov_;   //cover limit to use a contig
unsigned int minsize_;


public:
	Sam2alr();
	virtual ~Sam2alr();

	/*******************************************************************************/
	/*run : */
	/*******************************************************************************/
	void run(ifstream & reffile, ifstream & samfile, ofstream & out, ofstream & fasta, ofstream & covfile);
	void run(ifstream & reffile, ofstream & out, ofstream & fasta, ofstream & covfile);

	int readNames(ifstream & filenames);
	void addSamFile(string file){samfiles_.push_back(file);}

	void setCoverThreshold(double d){mincov_ = d;}
	double getCoverThreshold(){return mincov_;}

	void setSizeThreshold(unsigned int i){minsize_ = i;}
	unsigned int getSizeThreshold(){return minsize_;}

	/*******************************************************************************/
	/*ERROR messages*/
	/*******************************************************************************/
	void error_wrongSamInputFormat();

private:
	/*******************************************************************************/
	/*clean : delete trees after each contig */
	/*******************************************************************************/
	void clean();

	/*******************************************************************************/
	/*getNodeByOrder : return nodes position */
	/*******************************************************************************/
	OSite* getNodeByOrder(unsigned int pos);
	OSite* getNodeByOrder(unsigned int pos, OSite * root);

	/*******************************************************************************/
	/*initialize : create main tree */
	/*******************************************************************************/
	void initialize(OSite * node, BasicSequence & seq);

	/*******************************************************************************/
	/*newTree : create individual tree */
	/*******************************************************************************/
	OSite* newTree(string indiv);

	/*******************************************************************************/
	/*samline2OSiteStrain : fill tree with a samline */
	/*******************************************************************************/
	void samline2OSiteStrain(Sam::Samline & saml, OSite * node, unsigned int index);

	/*******************************************************************************/
	/*writeContig : write corrected contig sequence */
	/*******************************************************************************/
	void writeContig(ostream & out);

	/*******************************************************************************/
	/*writeAlignment : DEBUG display read alignment !increase time execution!*/
	/*******************************************************************************/
	void writeAlignment(OSite * root, unsigned int nbseq);




};

#endif /* SAM2ALR_H_ */

}
